@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 u"""
-read_tide_model.py (09/2020)
+read_tide_model.py (03/2021)
 Reads files for a tidal model and makes initial calculations to run tide program
 Includes functions to extract tidal harmonic constants from OTIS tide models for
     given locations
@@ -28,6 +28,7 @@ OPTIONS:
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
         linear, nearest: scipy regular grid interpolations
+    EXTRAPOLATE: extrapolate model using nearest-neighbors
     GRID: binary file type to read
         ATLAS: reading a global solution with localized solutions
         OTIS: combined global solution
@@ -47,9 +48,17 @@ PYTHON DEPENDENCIES:
 
 PROGRAM DEPENDENCIES:
     convert_ll_xy.py: converts lat/lon points to and from projected coordinates
-    bilinear_interp.py: bilinear interpolation of data to specified coordinates
+    bilinear_interp.py: bilinear interpolation of data to coordinates
+    nearest_extrap.py: nearest-neighbor extrapolation of data to coordinates
 
 UPDATE HISTORY:
+    Updated 03/2021: add extrapolation check where there are no invalid points
+        prevent ComplexWarning for fill values when calculating amplitudes
+        can read from single constituent TPXO9 ATLAS binary files
+        replaced numpy bool/int to prevent deprecation warnings
+    Updated 02/2021: set invalid values to nan in extrapolation
+        replaced numpy bool to prevent deprecation warning
+    Updated 12/2020: added valid data extrapolation with nearest_extrap
     Updated 09/2020: set bounds error to false for regular grid interpolations
         adjust dimensions of input coordinates to be iterable
         use masked arrays with atlas models and grids. make 2' grid with nearest
@@ -73,10 +82,12 @@ import numpy as np
 import scipy.interpolate
 from pyTMD.convert_ll_xy import convert_ll_xy
 from pyTMD.bilinear_interp import bilinear_interp
+from pyTMD.nearest_extrap import nearest_extrap
+import matplotlib.pyplot as plt
 
 #-- PURPOSE: extract tidal harmonic constants from tide models at coordinates
 def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
-    METHOD='spline', GRID='OTIS'):
+    METHOD='spline', EXTRAPOLATE=False, GRID='OTIS'):
     """
     Reads files for an OTIS-formatted tidal model
     Makes initial calculations to run the tide program
@@ -89,17 +100,20 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
     grid_file: grid file for model
     model_file: model file containing each constituent
     EPSG: projection of tide model data
-    TYPE: tidal variable to run
-        z: heights
-        u: horizontal transport velocities
-        v: vertical transport velocities
 
     Keyword arguments
     -----------------
+    TYPE: tidal variable to read
+        z: heights
+        u: horizontal transport velocities
+        U: horizontal depth-averaged transport
+        v: vertical transport velocities
+        V: vertical depth-averaged transport
     METHOD: interpolation method
         bilinear: quick bilinear interpolation
         spline: scipy bivariate spline interpolation
         linear, nearest: scipy regular grid interpolations
+    EXTRAPOLATE: extrapolate model using nearest-neighbors
     GRID: binary file type to read
         ATLAS: reading a global solution with localized solutions
         OTIS: combined global solution
@@ -123,7 +137,6 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
     #-- adjust dimensions of input coordinates to be iterable
     #-- run wrapper function to convert coordinate systems of input lat/lon
     x,y = convert_ll_xy(np.atleast_1d(ilon),np.atleast_1d(ilat),EPSG,'F')
-    invalid = (x < xi.min()) | (x > xi.max()) | (y < yi.min()) | (y > yi.max())
     #-- grid step size of tide model
     dx = xi[1] - xi[0]
     dy = yi[1] - yi[0]
@@ -150,6 +163,8 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
     if (np.max(x) > np.max(xi)) & (EPSG == '4326'):
         gt180, = np.nonzero(x > 180)
         x[gt180] -= 360.0
+    #-- determine if any input points are outside of the model bounds
+    invalid = (x < xi.min()) | (x > xi.max()) | (y < yi.min()) | (y > yi.max())
 
     #-- masks zero values
     hz = np.ma.array(hz,mask=(hz==0))
@@ -169,23 +184,23 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
         #-- use quick bilinear to interpolate values
         D = bilinear_interp(xi,yi,hz,x,y)
         mz1 = bilinear_interp(xi,yi,mz,x,y)
-        mz1 = np.ceil(mz1).astype(mz.dtype)
+        mz1 = np.floor(mz1).astype(mz.dtype)
         if (TYPE != 'z'):
             mu1 = bilinear_interp(xi,yi,mu,x,y)
-            mu1 = np.ceil(mu1).astype(mu.dtype)
+            mu1 = np.floor(mu1).astype(mu.dtype)
             mv1 = bilinear_interp(xi,yi,mv,x,y)
-            mv1 = np.ceil(mv1).astype(mz.dtype)
+            mv1 = np.floor(mv1).astype(mz.dtype)
     elif (METHOD == 'spline'):
         #-- use scipy bivariate splines to interpolate values
         f1=scipy.interpolate.RectBivariateSpline(xi,yi,hz.T,kx=1,ky=1)
         f2=scipy.interpolate.RectBivariateSpline(xi,yi,mz.T,kx=1,ky=1)
         D = f1.ev(x,y)
-        mz1 = np.ceil(f2.ev(x,y)).astype(mz.dtype)
+        mz1 = np.floor(f2.ev(x,y)).astype(mz.dtype)
         if (TYPE != 'z'):
             f3=scipy.interpolate.RectBivariateSpline(xi,yi,mu.T,kx=1,ky=1)
             f4=scipy.interpolate.RectBivariateSpline(xi,yi,mv.T,kx=1,ky=1)
-            mu1 = np.ceil(f3.ev(x,y)).astype(mu.dtype)
-            mv1 = np.ceil(f4.ev(x,y)).astype(mv.dtype)
+            mu1 = np.floor(f3.ev(x,y)).astype(mu.dtype)
+            mv1 = np.floor(f4.ev(x,y)).astype(mv.dtype)
     else:
         #-- use scipy regular grid to interpolate values for a given method
         r1 = scipy.interpolate.RegularGridInterpolator((yi,xi),hz,
@@ -193,14 +208,14 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
         r2 = scipy.interpolate.RegularGridInterpolator((yi,xi),mz,
             method=METHOD,bounds_error=False,fill_value=0)
         D = r1.__call__(np.c_[y,x])
-        mz1 = np.ceil(r2.__call__(np.c_[y,x])).astype(mz.dtype)
+        mz1 = np.floor(r2.__call__(np.c_[y,x])).astype(mz.dtype)
         if (TYPE != 'z'):
             r3 = scipy.interpolate.RegularGridInterpolator((yi,xi),mu,
                 method=METHOD,bounds_error=False,fill_value=0)
             r4 = scipy.interpolate.RegularGridInterpolator((yi,xi),mv,
                 method=METHOD,bounds_error=False,fill_value=0)
-            mu1 = np.ceil(r3.__call__(np.c_[y,x])).astype(mu.dtype)
-            mv1 = np.ceil(r4.__call__(np.c_[y,x])).astype(mv.dtype)
+            mu1 = np.floor(r3.__call__(np.c_[y,x])).astype(mu.dtype)
+            mv1 = np.floor(r4.__call__(np.c_[y,x])).astype(mv.dtype)
 
     #-- u and v are velocities in cm/s
     if TYPE in ('v','u'):
@@ -210,18 +225,25 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
         unit_conv = 1.0
 
     #-- read and interpolate each constituent
-    constituents,nc = read_constituents(model_file)
+    if isinstance(model_file,list):
+        constituents = [read_constituents(m)[0].pop() for m in model_file]
+        nc = len(constituents)
+    else:
+        constituents,nc = read_constituents(model_file)
+    #-- number of output data points
     npts = len(D)
     amplitude = np.ma.zeros((npts,nc))
-    amplitude.mask = np.zeros((npts,nc),dtype=np.bool)
+    amplitude.mask = np.zeros((npts,nc),dtype=bool)
     ph = np.ma.zeros((npts,nc))
-    ph.mask = np.zeros((npts,nc),dtype=np.bool)
+    ph.mask = np.zeros((npts,nc),dtype=bool)
     for i,c in enumerate(constituents):
         if (TYPE == 'z'):
             #-- read constituent from elevation file
             if (GRID == 'ATLAS'):
                 z0,zlocal = read_atlas_elevation(model_file,i,c)
                 xi,yi,z=combine_atlas_model(x0,y0,z0,pmask,zlocal,VARIABLE='z')
+            elif isinstance(model_file,list):
+                z = read_elevation_file(model_file[i],0)
             else:
                 z = read_elevation_file(model_file,i)
             #-- replace original values with extend matrices
@@ -235,7 +257,7 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                 #-- use quick bilinear to interpolate values
                 z1.data[:] = bilinear_interp(xi,yi,z,x,y,dtype=np.complex128)
                 #-- replace nan values with fill_value
-                z1.mask = (np.isnan(z1.data) | (~mz1.astype(np.bool)))
+                z1.mask = (np.isnan(z1.data) | (~mz1.astype(bool)))
                 z1.data[z1.mask] = z1.fill_value
             elif (METHOD == 'spline'):
                 #-- use scipy bivariate splines to interpolate values
@@ -246,7 +268,7 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                 z1.data.real = f1.ev(x,y)
                 z1.data.imag = f2.ev(x,y)
                 #-- replace zero values with fill_value
-                z1.mask = (~mz1.astype(np.bool))
+                z1.mask = (~mz1.astype(bool))
                 z1.data[z1.mask] = z1.fill_value
             else:
                 #-- use scipy regular grid to interpolate values
@@ -255,7 +277,19 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                 z1 = np.ma.zeros((npts),dtype=z.dtype)
                 z1.data[:] = r1.__call__(np.c_[y,x])
                 #-- replace invalid values with fill_value
-                z1.mask = (z1.data == z1.fill_value) | (~mz1.astype(np.bool))
+                z1.mask = (z1.data == z1.fill_value) | (~mz1.astype(bool))
+                z1.data[z1.mask] = z1.fill_value
+            #-- extrapolate data using nearest-neighbors
+            if EXTRAPOLATE and np.any(z1.mask):
+                #-- find invalid data points
+                inv, = np.nonzero(z1.mask)
+                #-- replace zero values with nan
+                z[z==0] = np.nan
+                #-- extrapolate points within 10km of valid model points
+                z1.data[inv] = nearest_extrap(xi,yi,z,x[inv],y[inv],
+                    dtype=np.complex128,cutoff=10.0,EPSG=EPSG)
+                #-- replace nan values with fill_value
+                z1.mask[inv] = np.isnan(z1.data[inv])
                 z1.data[z1.mask] = z1.fill_value
             #-- amplitude and phase of the constituent
             amplitude.data[:,i] = np.abs(z1.data)
@@ -267,6 +301,8 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
             if (GRID == 'ATLAS'):
                 u0,v0,uvlocal = read_atlas_transport(model_file,i,c)
                 xi,yi,u=combine_atlas_model(x0,y0,u0,pmask,uvlocal,VARIABLE='u')
+            elif isinstance(model_file,list):
+                u,v = read_transport_file(model_file[i],0)
             else:
                 u,v = read_transport_file(model_file,i)
             #-- replace original values with extend matrices
@@ -282,7 +318,7 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                 #-- use quick bilinear to interpolate values
                 u1.data[:] = bilinear_interp(xu,yi,u,x,y,dtype=np.complex128)
                 #-- replace nan values with fill_value
-                u1.mask = (np.isnan(u1.data) | (~mu1.astype(np.bool)))
+                u1.mask = (np.isnan(u1.data) | (~mu1.astype(bool)))
                 u1.data[u1.mask] = u1.fill_value
             elif (METHOD == 'spline'):
                 f1 = scipy.interpolate.RectBivariateSpline(xu,yi,
@@ -292,7 +328,7 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                 u1.data.real = f1.ev(x,y)
                 u1.data.imag = f2.ev(x,y)
                 #-- replace zero values with fill_value
-                u1.mask = (~mu1.astype(np.bool))
+                u1.mask = (~mu1.astype(bool))
                 u1.data[u1.mask] = u1.fill_value
             else:
                 #-- use scipy regular grid to interpolate values
@@ -300,12 +336,23 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                     method=METHOD,bounds_error=False,fill_value=u1.fill_value)
                 u1.data[:] = r1.__call__(np.c_[y,x])
                 #-- replace invalid values with fill_value
-                u1.mask = (u1.data == u1.fill_value) | (~mu1.astype(np.bool))
+                u1.mask = (u1.data == u1.fill_value) | (~mu1.astype(bool))
+                u1.data[u1.mask] = u1.fill_value
+            #-- extrapolate data using nearest-neighbors
+            if EXTRAPOLATE and np.any(u1.mask):
+                #-- find invalid data points
+                inv, = np.nonzero(u1.mask)
+                #-- replace zero values with nan
+                u[u==0] = np.nan
+                #-- extrapolate points within 10km of valid model points
+                u1.data[inv] = nearest_extrap(xu,yi,u,x[inv],y[inv],
+                    dtype=np.complex128,cutoff=10.0,EPSG=EPSG)
+                #-- replace nan values with fill_value
+                u1.mask[inv] = np.isnan(u1.data[inv])
                 u1.data[u1.mask] = u1.fill_value
             #-- convert units
-            u1 = u1/unit_conv
             #-- amplitude and phase of the constituent
-            amplitude.data[:,i] = np.abs(u1)
+            amplitude.data[:,i] = np.abs(u1.data)/unit_conv
             amplitude.mask[:,i] = np.copy(u1.mask)
             ph.data[:,i] = np.arctan2(-np.imag(u1),np.real(u1))
             ph.mask[:,i] = np.copy(u1.mask)
@@ -314,6 +361,8 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
             if (GRID == 'ATLAS'):
                 u0,v0,uvlocal = read_atlas_transport(model_file,i,c)
                 xi,yi,v = combine_atlas_model(x0,y0,v0,pmask,local,VARIABLE='v')
+            elif isinstance(model_file,list):
+                u,v = read_transport_file(model_file[i],0)
             else:
                 u,v = read_transport_file(model_file,i)
             #-- replace original values with extend matrices
@@ -329,7 +378,7 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                 #-- use quick bilinear to interpolate values
                 v1.data = bilinear_interp(xi,yv,v,x,y,dtype=np.complex128)
                 #-- replace nan values with fill_value
-                v1.mask = (np.isnan(v1.data) | (~mv1.astype(np.bool)))
+                v1.mask = (np.isnan(v1.data) | (~mv1.astype(bool)))
                 v1.data[v1.mask] = v1.fill_value
             elif (METHOD == 'spline'):
                 f1 = scipy.interpolate.RectBivariateSpline(xi,yv,
@@ -339,7 +388,7 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                 v1.data.real = f1.ev(x,y)
                 v1.data.imag = f2.ev(x,y)
                 #-- replace zero values with fill_value
-                v1.mask = (~mv1.astype(np.bool))
+                v1.mask = (~mv1.astype(bool))
                 v1.data[v1.mask] = v1.fill_value
             else:
                 #-- use scipy regular grid to interpolate values
@@ -347,12 +396,23 @@ def extract_tidal_constants(ilon, ilat, grid_file, model_file, EPSG, TYPE='z',
                     method=METHOD,bounds_error=False,fill_value=v1.fill_value)
                 v1.data[:] = r1.__call__(np.c_[y,x])
                 #-- replace invalid values with fill_value
-                v1.mask = (v1.data == v1.fill_value) | (~mv1.astype(np.bool))
+                v1.mask = (v1.data == v1.fill_value) | (~mv1.astype(bool))
+                v1.data[v1.mask] = v1.fill_value
+            #-- extrapolate data using nearest-neighbors
+            if EXTRAPOLATE and np.any(v1.mask):
+                #-- find invalid data points
+                inv, = np.nonzero(v1.mask)
+                #-- replace zero values with nan
+                v[z==v] = np.nan
+                #-- extrapolate points within 10km of valid model points
+                v1.data[inv] = nearest_extrap(x,yv,v,x[inv],y[inv],
+                    dtype=np.complex128,cutoff=10.0,EPSG=EPSG)
+                #-- replace nan values with fill_value
+                v1.mask[inv] = np.isnan(v1.data[inv])
                 v1.data[v1.mask] = v1.fill_value
             #-- convert units
-            v1 = v1/unit_conv
             #-- amplitude and phase of the constituent
-            amplitude.data[:,i] = np.abs(v1)
+            amplitude.data[:,i] = np.abs(v1.data)/unit_conv
             amplitude.mask[:,i] = np.copy(v1.mask)
             ph.data[:,i] = np.arctan2(-np.imag(v1),np.real(v1))
             ph.mask[:,i] = np.copy(v1.mask)
@@ -414,7 +474,7 @@ def extend_matrix(input_matrix):
 #-- PURPOSE: read tide grid file
 def read_tide_grid(input_file):
     """
-    Read grid file to extract model coordinates, bathymety, masks and indices
+    Read grid file to extract model coordinates, bathymetry, masks and indices
 
     Arguments
     ---------
@@ -424,7 +484,7 @@ def read_tide_grid(input_file):
     -------
     x: x-coordinates of input grid
     y: y-coordinates of input grid
-    hz: model bathymety
+    hz: model bathymetry
     mz: land/water mask
     iob: open boundary index
     dt: time step
@@ -470,7 +530,7 @@ def read_tide_grid(input_file):
 #-- PURPOSE: read tide grid file with localized solutions
 def read_atlas_grid(input_file):
     """
-    Read ATLAS grid file to extract model coordinates, bathymety, masks and
+    Read ATLAS grid file to extract model coordinates, bathymetry, masks and
     indices for both global and local solutions
 
     Arguments
@@ -481,13 +541,13 @@ def read_atlas_grid(input_file):
     -------
     x: x-coordinates of input ATLAS grid
     y: y-coordinates of input ATLAS grid
-    hz: model bathymety
+    hz: model bathymetry
     mz: land/water mask
     iob: open boundary index
     dt: time step
     pmask: global mask
     local: dictionary of local tidal solutions for grid variables
-        depth: model bathymety
+        depth: model bathymetry
     """
     #-- read the input file to get file information
     fd = os.open(os.path.expanduser(input_file),os.O_RDONLY)
@@ -547,7 +607,7 @@ def read_atlas_grid(input_file):
         jz = np.fromfile(fid, dtype=np.dtype('>i4'), count=nd)
         fid.seek(8,1)
         depth = np.ma.zeros((ny1,nx1))
-        depth.mask = np.ones((ny1,nx1),dtype=np.bool)
+        depth.mask = np.ones((ny1,nx1),dtype=bool)
         depth.data[jz-1,iz-1] = np.fromfile(fid,dtype=np.dtype('>f4'),count=nd)
         depth.mask[jz-1,iz-1] = False
         fid.seek(4,1)
@@ -608,7 +668,7 @@ def read_elevation_file(input_file,ic):
     fid.seek(nskip,1)
     #-- real and imaginary components of elevation
     h = np.ma.zeros((ny,nx),dtype=np.complex64)
-    h.mask = np.zeros((ny,nx),dtype=np.bool)
+    h.mask = np.zeros((ny,nx),dtype=bool)
     for i in range(ny):
         temp = np.fromfile(fid, dtype=np.dtype('>f4'), count=2*nx)
         h.data.real[i,:] = temp[0:2*nx-1:2]
@@ -651,7 +711,7 @@ def read_atlas_elevation(input_file,ic,constituent):
     fid.seek(nskip,1)
     #-- real and imaginary components of elevation
     h = np.ma.zeros((ny,nx),dtype=np.complex64)
-    h.mask = np.zeros((ny,nx),dtype=np.bool)
+    h.mask = np.zeros((ny,nx),dtype=bool)
     for i in range(ny):
         temp = np.fromfile(fid, dtype=np.dtype('>f4'), count=2*nx)
         h.data.real[i,:] = temp[0:2*nx-1:2]
@@ -690,7 +750,7 @@ def read_atlas_elevation(input_file,ic,constituent):
             fid.seek(nskip,1)
             #-- real and imaginary components of elevation
             h1 = np.ma.zeros((ny1,nx1),fill_value=np.nan,dtype=np.complex64)
-            h1.mask = np.ones((ny1,nx1),dtype=np.bool)
+            h1.mask = np.ones((ny1,nx1),dtype=bool)
             temp = np.fromfile(fid, dtype=np.dtype('>f4'), count=2*nz)
             h1.data.real[jz-1,iz-1] = temp[0:2*nz-1:2]
             h1.data.imag[jz-1,iz-1] = temp[1:2*nz:2]
@@ -737,9 +797,9 @@ def read_transport_file(input_file,ic):
     fid.seek(nskip,1)
     #-- real and imaginary components of transport
     u = np.ma.zeros((ny,nx),dtype=np.complex64)
-    u.mask = np.zeros((ny,nx),dtype=np.bool)
+    u.mask = np.zeros((ny,nx),dtype=bool)
     v = np.ma.zeros((ny,nx),dtype=np.complex64)
-    v.mask = np.zeros((ny,nx),dtype=np.bool)
+    v.mask = np.zeros((ny,nx),dtype=bool)
     for i in range(ny):
         temp = np.fromfile(fid, dtype=np.dtype('>f4'), count=4*nx)
         u.data.real[i,:] = temp[0:4*nx-3:4]
@@ -786,9 +846,9 @@ def read_atlas_transport(input_file,ic,constituent):
     fid.seek(nskip,1)
     #-- real and imaginary components of transport
     u = np.ma.zeros((ny,nx),dtype=np.complex64)
-    u.mask = np.zeros((ny,nx),dtype=np.bool)
+    u.mask = np.zeros((ny,nx),dtype=bool)
     v = np.ma.zeros((ny,nx),dtype=np.complex64)
-    v.mask = np.zeros((ny,nx),dtype=np.bool)
+    v.mask = np.zeros((ny,nx),dtype=bool)
     for i in range(ny):
         temp = np.fromfile(fid, dtype=np.dtype('>f4'), count=4*nx)
         u.data.real[i,:] = temp[0:4*nx-3:4]
@@ -833,7 +893,7 @@ def read_atlas_transport(input_file,ic,constituent):
             fid.seek(nskip,1)
             #-- real and imaginary components of u transport
             u1 = np.ma.zeros((ny1,nx1),fill_value=np.nan,dtype=np.complex64)
-            u1.mask = np.ones((ny1,nx1),dtype=np.bool)
+            u1.mask = np.ones((ny1,nx1),dtype=bool)
             tmpu = np.fromfile(fid, dtype=np.dtype('>f4'), count=2*nu)
             u1.data.real[ju-1,iu-1] = tmpu[0:2*nu-1:2]
             u1.data.imag[ju-1,iu-1] = tmpu[1:2*nu:2]
@@ -841,7 +901,7 @@ def read_atlas_transport(input_file,ic,constituent):
             fid.seek(8,1)
             #-- real and imaginary components of v transport
             v1 = np.ma.zeros((ny1,nx1),fill_value=np.nan,dtype=np.complex64)
-            v1.mask = np.ones((ny1,nx1),dtype=np.bool)
+            v1.mask = np.ones((ny1,nx1),dtype=bool)
             tmpv = np.fromfile(fid, dtype=np.dtype('>f4'), count=2*nv)
             v1.data.real[jv-1,iv-1] = tmpv[0:2*nv-1:2]
             v1.data.imag[jv-1,iv-1] = tmpv[1:2*nv:2]
@@ -875,7 +935,7 @@ def create_atlas_mask(xi,yi,mz,local,VARIABLE=None):
     Keyword arguments
     -----------------
     VARIABLE: key for variable within each local solution
-        depth: model bathymety
+        depth: model bathymetry
 
     Returns
     -------
@@ -905,8 +965,8 @@ def create_atlas_mask(xi,yi,mz,local,VARIABLE=None):
         for indy,indx in zip(validy,validx):
             #-- check if model is -180:180
             lon30 = (X[indy,indx]+360.) if (X[indy,indx]<=0.0) else X[indy,indx]
-            ii = np.int((lon30 - x30[0])/d30)
-            jj = np.int((Y[indy,indx] - y30[0])/d30)
+            ii = int((lon30 - x30[0])/d30)
+            jj = int((Y[indy,indx] - y30[0])/d30)
             #-- fill global mask with regional solution
             m30[jj,ii] = 1
     #-- return the 2 arc-minute mask
@@ -930,7 +990,7 @@ def combine_atlas_model(xi,yi,zi,pmask,local,VARIABLE=None):
     Keyword arguments
     -----------------
     VARIABLE: key for variable within each local solution
-        depth: model bathymety
+        depth: model bathymetry
         z: tidal elevation
         u: zonal tidal transport
         v: meridional zonal transport
@@ -947,7 +1007,7 @@ def combine_atlas_model(xi,yi,zi,pmask,local,VARIABLE=None):
     y30 = np.arange(-90.0+d30/2.0, 90.0+d30/2.0, d30)
     #-- interpolate global solution to 2 arc-minute solution
     z30 = np.ma.zeros((len(y30),len(x30)),dtype=zi.dtype)
-    z30.mask = np.zeros((len(y30),len(x30)),dtype=np.bool)
+    z30.mask = np.zeros((len(y30),len(x30)),dtype=bool)
     #-- test if combining elevation/transport variables with complex components
     if np.iscomplexobj(z30):
         f1 = scipy.interpolate.RectBivariateSpline(xi, yi, zi.real.T, kx=1,ky=1)
@@ -969,8 +1029,8 @@ def combine_atlas_model(xi,yi,zi,pmask,local,VARIABLE=None):
         for indy,indx in zip(validy,validx):
             #-- check if model is -180:180
             lon30 = (X[indy,indx]+360.) if (X[indy,indx]<=0.0) else X[indy,indx]
-            ii = np.int((lon30 - x30[0])/d30)
-            jj = np.int((Y[indy,indx] - y30[0])/d30)
+            ii = int((lon30 - x30[0])/d30)
+            jj = int((Y[indy,indx] - y30[0])/d30)
             #-- fill global model with regional solution
             z30.data[jj,ii] = zlocal[indy,indx]
     #-- return 2 arc-minute solution and coordinates
@@ -983,35 +1043,35 @@ def Muv(hz):
     Construct masks for zeta, u and v nodes on a C-grid
     """
     ny,nx = np.shape(hz)
-    mz = (hz > 0).astype(np.int)
+    mz = (hz > 0).astype(int)
     #-- x-indices
-    indx = np.zeros((nx),dtype=np.int)
+    indx = np.zeros((nx),dtype=int)
     indx[:-1] = np.arange(1,nx)
     indx[-1] = 0
     #-- y-indices
-    indy = np.zeros((ny),dtype=np.int)
+    indy = np.zeros((ny),dtype=int)
     indy[:-1] = np.arange(1,ny)
     indy[-1] = 0
     #-- calculate mu and mv
-    mu = np.zeros((ny,nx),dtype=np.int)
-    mv = np.zeros((ny,nx),dtype=np.int)
+    mu = np.zeros((ny,nx),dtype=int)
+    mv = np.zeros((ny,nx),dtype=int)
     mu[indy,:] = mz*mz[indy,:]
     mv[:,indx] = mz*mz[:,indx]
     return (mu,mv,mz)
 
-#-- PURPOSE: Interpolate bathymety to zeta, u and v nodes on a C-grid
+#-- PURPOSE: Interpolate bathymetry to zeta, u and v nodes on a C-grid
 def Huv(hz):
     """
-    Interpolate bathymety to zeta, u and v nodes on a C-grid
+    Interpolate bathymetry to zeta, u and v nodes on a C-grid
     """
     ny,nx = np.shape(hz)
     mu,mv,mz = Muv(hz)
     #-- x-indices
-    indx = np.zeros((nx),dtype=np.int)
+    indx = np.zeros((nx),dtype=int)
     indx[0] = nx-1
     indx[1:] = np.arange(1,nx)
     #-- y-indices
-    indy = np.zeros((ny),dtype=np.int)
+    indy = np.zeros((ny),dtype=int)
     indy[0] = ny-1
     indy[1:] = np.arange(1,ny)
     #-- calculate hu and hv
