@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_LPET_ICESat2_ATL03.py
-Written by Tyler Sutterley (11/2020)
+Written by Tyler Sutterley (04/2021)
 Calculates long-period equilibrium tidal elevations for correcting ICESat-2
     geolocated photon height data
 Will calculate the long-period tides for all ATL03 segments and not just ocean
@@ -28,11 +28,14 @@ PROGRAM DEPENDENCIES:
     read_ICESat2_ATL03.py: reads ICESat-2 global geolocated photon data files
     time.py: utilities for calculating time operations
     utilities: download and management utilities for syncing files
-    convert_julian.py: returns the calendar date and time given a Julian date
     calc_delta_time.py: calculates difference between universal and dynamic time
     compute_equilibrium_tide.py: calculates long-period equilibrium ocean tides
 
 UPDATE HISTORY:
+    Updated 04/2021: can use a generically named ATL03 file as input
+    Updated 03/2021: replaced numpy bool/int to prevent deprecation warnings
+    Updated 12/2020: H5py deprecation warning change to use make_scale
+        merged time conversion routines into module
     Written 11/2020
 """
 from __future__ import print_function
@@ -46,7 +49,6 @@ import datetime
 import numpy as np
 import pyTMD.time
 from pyTMD.utilities import get_data_path
-from pyTMD.convert_julian import convert_julian
 from pyTMD.calc_delta_time import calc_delta_time
 from pyTMD.compute_equilibrium_tide import compute_equilibrium_tide
 from icesat2_toolkit.read_ICESat2_ATL03 import read_HDF5_ATL03_main, \
@@ -54,17 +56,27 @@ from icesat2_toolkit.read_ICESat2_ATL03 import read_HDF5_ATL03_main, \
 
 #-- PURPOSE: read ICESat-2 geolocated photon data (ATL03) from NSIDC
 #-- compute long-period equilibrium tides at points and times
-def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
+def compute_LPET_ICESat2(INPUT_FILE, VERBOSE=False, MODE=0o775):
 
-    #-- read data from FILE
-    print('{0} -->'.format(os.path.basename(FILE))) if VERBOSE else None
-    IS2_atl03_mds,IS2_atl03_attrs,IS2_atl03_beams = read_HDF5_ATL03_main(FILE,
+    #-- read data from input file
+    print('{0} -->'.format(os.path.basename(INPUT_FILE))) if VERBOSE else None
+    IS2_atl03_mds,IS2_atl03_attrs,IS2_atl03_beams = read_HDF5_ATL03_main(INPUT_FILE,
         ATTRIBUTES=True)
-    DIRECTORY = os.path.dirname(FILE)
+    DIRECTORY = os.path.dirname(INPUT_FILE)
     #-- extract parameters from ICESat-2 ATLAS HDF5 file name
     rx = re.compile(r'(processed_)?(ATL\d{2})_(\d{4})(\d{2})(\d{2})(\d{2})'
-        '(\d{2})(\d{2})_(\d{4})(\d{2})(\d{2})_(\d{3})_(\d{2})(.*?).h5$')
-    SUB,PRD,YY,MM,DD,HH,MN,SS,TRK,CYCL,GRAN,RL,VERS,AUX = rx.findall(FILE).pop()
+        r'(\d{2})(\d{2})_(\d{4})(\d{2})(\d{2})_(\d{3})_(\d{2})(.*?).h5$')
+    try:
+        SUB,PRD,YY,MM,DD,HH,MN,SS,TRK,CYCL,GRAN,RL,VERS,AUX = rx.findall(INPUT_FILE).pop()
+    except:
+        #-- output long-period equilibrium tide HDF5 file (generic)
+        fileBasename,fileExtension = os.path.splitext(INPUT_FILE)
+        OUTPUT_FILE = '{0}_{1}{2}'.format(fileBasename,'LPET',fileExtension)
+    else:
+        #-- output long-period equilibrium tide HDF5 file for ASAS/NSIDC granules
+        args = (PRD,YY,MM,DD,HH,MN,SS,TRK,CYCL,GRAN,RL,VERS,AUX)
+        file_format = '{0}_LPET_{1}{2}{3}{4}{5}{6}_{7}{8}{9}_{10}_{11}{12}.h5'
+        OUTPUT_FILE = file_format.format(*args)
 
     #-- number of GPS seconds between the GPS epoch
     #-- and ATLAS Standard Data Product (SDP) epoch
@@ -73,6 +85,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
     #-- copy variables for outputting to HDF5 file
     IS2_atl03_tide = {}
     IS2_atl03_fill = {}
+    IS2_atl03_dims = {}
     IS2_atl03_tide_attrs = {}
     #-- number of GPS seconds between the GPS epoch (1980-01-06T00:00:00Z UTC)
     #-- and ATLAS Standard Data Product (SDP) epoch (2018-01-01T00:00:00Z UTC)
@@ -92,10 +105,11 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
         #-- output data dictionaries for beam
         IS2_atl03_tide[gtx] = dict(geolocation={}, geophys_corr={})
         IS2_atl03_fill[gtx] = dict(geolocation={}, geophys_corr={})
+        IS2_atl03_dims[gtx] = dict(geolocation={}, geophys_corr={})
         IS2_atl03_tide_attrs[gtx] = dict(geolocation={}, geophys_corr={})
 
         #-- read data and attributes for beam
-        val,attrs = read_HDF5_ATL03_beam(FILE,gtx,ATTRIBUTES=True)
+        val,attrs = read_HDF5_ATL03_beam(INPUT_FILE,gtx,ATTRIBUTES=True)
         #-- number of segments
         n_seg = len(val['geolocation']['segment_id'])
         #-- extract variables for computing equilibrium tides
@@ -145,6 +159,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
         #-- delta time in geolocation group
         IS2_atl03_tide[gtx]['geolocation']['delta_time'] = delta_time
         IS2_atl03_fill[gtx]['geolocation']['delta_time'] = None
+        IS2_atl03_dims[gtx]['geolocation']['delta_time'] = None
         IS2_atl03_tide_attrs[gtx]['geolocation']['delta_time'] = {}
         IS2_atl03_tide_attrs[gtx]['geolocation']['delta_time']['units'] = "seconds since 2018-01-01"
         IS2_atl03_tide_attrs[gtx]['geolocation']['delta_time']['long_name'] = "Elapsed GPS seconds"
@@ -162,6 +177,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
         #-- delta time in geophys_corr group
         IS2_atl03_tide[gtx]['geophys_corr']['delta_time'] = delta_time
         IS2_atl03_fill[gtx]['geophys_corr']['delta_time'] = None
+        IS2_atl03_dims[gtx]['geophys_corr']['delta_time'] = None
         IS2_atl03_tide_attrs[gtx]['geophys_corr']['delta_time'] = {}
         IS2_atl03_tide_attrs[gtx]['geophys_corr']['delta_time']['units'] = "seconds since 2018-01-01"
         IS2_atl03_tide_attrs[gtx]['geophys_corr']['delta_time']['long_name'] = "Elapsed GPS seconds"
@@ -180,6 +196,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
         #-- latitude
         IS2_atl03_tide[gtx]['geolocation']['reference_photon_lat'] = lat
         IS2_atl03_fill[gtx]['geolocation']['reference_photon_lat'] = None
+        IS2_atl03_dims[gtx]['geolocation']['reference_photon_lat'] = ['delta_time']
         IS2_atl03_tide_attrs[gtx]['geolocation']['reference_photon_lat'] = {}
         IS2_atl03_tide_attrs[gtx]['geolocation']['reference_photon_lat']['units'] = "degrees_north"
         IS2_atl03_tide_attrs[gtx]['geolocation']['reference_photon_lat']['contentType'] = "physicalMeasurement"
@@ -194,6 +211,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
         #-- longitude
         IS2_atl03_tide[gtx]['geolocation']['reference_photon_lon'] = lon
         IS2_atl03_fill[gtx]['geolocation']['reference_photon_lon'] = None
+        IS2_atl03_dims[gtx]['geolocation']['reference_photon_lon'] = ['delta_time']
         IS2_atl03_tide_attrs[gtx]['geolocation']['reference_photon_lon'] = {}
         IS2_atl03_tide_attrs[gtx]['geolocation']['reference_photon_lon']['units'] = "degrees_east"
         IS2_atl03_tide_attrs[gtx]['geolocation']['reference_photon_lon']['contentType'] = "physicalMeasurement"
@@ -208,6 +226,7 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
         #-- segment ID
         IS2_atl03_tide[gtx]['geolocation']['segment_id'] = segment_id
         IS2_atl03_fill[gtx]['geolocation']['segment_id'] = None
+        IS2_atl03_dims[gtx]['geolocation']['segment_id'] = ['delta_time']
         IS2_atl03_tide_attrs[gtx]['geolocation']['segment_id'] = {}
         IS2_atl03_tide_attrs[gtx]['geolocation']['segment_id']['units'] = "1"
         IS2_atl03_tide_attrs[gtx]['geolocation']['segment_id']['contentType'] = "referenceInformation"
@@ -218,9 +237,11 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
             "the second of the two 20m ATL03 segments included in the 40m ATL03 segment")
         IS2_atl03_tide_attrs[gtx]['geolocation']['segment_id']['coordinates'] = \
             "delta_time reference_photon_lat reference_photon_lon"
+
         #-- computed long-period equilibrium tide
         IS2_atl03_tide[gtx]['geophys_corr']['tide_equilibrium'] = tide_lpe
         IS2_atl03_fill[gtx]['geophys_corr']['tide_equilibrium'] = None
+        IS2_atl03_dims[gtx]['geophys_corr']['tide_equilibrium'] = ['delta_time']
         IS2_atl03_tide_attrs[gtx]['geophys_corr']['tide_equilibrium'] = {}
         IS2_atl03_tide_attrs[gtx]['geophys_corr']['tide_equilibrium']['units'] = "meters"
         IS2_atl03_tide_attrs[gtx]['geophys_corr']['tide_equilibrium']['contentType'] = "referenceInformation"
@@ -230,26 +251,22 @@ def compute_LPET_ICESat2(FILE, VERBOSE=False, MODE=0o775):
             "equilibrium tidal elevation from the summation of fifteen tidal spectral lines")
         IS2_atl03_tide_attrs[gtx]['geophys_corr']['tide_equilibrium']['reference'] = \
             "https://doi.org/10.1111/j.1365-246X.1973.tb03420.x"
-        IS2_atl03_tide_attrs[gtx]['geophys_corr']['tide_equilibrium']['coordinates'] = \
-            "segment_id delta_time reference_photon_lat reference_photon_lon"
         IS2_atl03_tide_attrs[gtx]['geophys_corr']['tide_equilibrium']['coordinates'] = \
             ("../geolocation/segment_id ../geolocation/delta_time "
             "../geolocation/reference_photon_lat ../geolocation/reference_photon_lon")
 
-    #-- output equilibrium tide HDF5 file
-    args = (PRD,YY,MM,DD,HH,MN,SS,TRK,CYCL,GRAN,RL,VERS,AUX)
-    file_format = '{0}_LPET_{1}{2}{3}{4}{5}{6}_{7}{8}{9}_{10}_{11}{12}.h5'
     #-- print file information
-    print('\t{0}'.format(file_format.format(*args))) if VERBOSE else None
-    HDF5_ATL03_tide_write(IS2_atl03_tide, IS2_atl03_tide_attrs, CLOBBER=True,
-        INPUT=os.path.basename(FILE), FILL_VALUE=IS2_atl03_fill,
-        FILENAME=os.path.join(DIRECTORY,file_format.format(*args)))
+    print('\t{0}'.format(OUTPUT_FILE)) if VERBOSE else None
+    HDF5_ATL03_tide_write(IS2_atl03_tide, IS2_atl03_tide_attrs,
+        CLOBBER=True, INPUT=os.path.basename(INPUT_FILE),
+        FILL_VALUE=IS2_atl03_fill, DIMENSIONS=IS2_atl03_dims,
+        FILENAME=os.path.join(DIRECTORY,OUTPUT_FILE))
     #-- change the permissions mode
-    os.chmod(os.path.join(DIRECTORY,file_format.format(*args)), MODE)
+    os.chmod(os.path.join(DIRECTORY,OUTPUT_FILE), MODE)
 
 #-- PURPOSE: outputting the tide values for ICESat-2 data to HDF5
 def HDF5_ATL03_tide_write(IS2_atl03_tide, IS2_atl03_attrs, INPUT=None,
-    FILENAME='', FILL_VALUE=None, CLOBBER=False):
+    FILENAME='', FILL_VALUE=None, DIMENSIONS=None, CLOBBER=False):
     #-- setting HDF5 clobber attribute
     if CLOBBER:
         clobber = 'w'
@@ -292,33 +309,30 @@ def HDF5_ATL03_tide_write(IS2_atl03_tide, IS2_atl03_attrs, INPUT=None,
                 att_val = IS2_atl03_attrs[gtx][key][att_name]
                 fileID[gtx][key].attrs[att_name] = att_val
 
-            #-- delta_time in group
-            v = IS2_atl03_tide[gtx][key]['delta_time']
-            attrs = IS2_atl03_attrs[gtx][key]['delta_time']
-            fillvalue = FILL_VALUE[gtx][key]['delta_time']
-            #-- Defining the HDF5 dataset variables
-            val = '{0}/{1}/{2}'.format(gtx,key,'delta_time')
-            h5[gtx][key]['delta_time'] = fileID.create_dataset(val,np.shape(v),
-                data=v, dtype=v.dtype, fillvalue=fillvalue, compression='gzip')
-            #-- add HDF5 variable attributes
-            for att_name,att_val in attrs.items():
-                h5[gtx][key]['delta_time'].attrs[att_name] = att_val
-
-            #-- add other variables for group
+            #-- all variables for group
             groupkeys = set(IS2_atl03_tide[gtx][key].keys())-set(['delta_time'])
-            for k in sorted(groupkeys):
+            for k in ['delta_time',*sorted(groupkeys)]:
                 #-- values and attributes
                 v = IS2_atl03_tide[gtx][key][k]
                 attrs = IS2_atl03_attrs[gtx][key][k]
                 fillvalue = FILL_VALUE[gtx][key][k]
                 #-- Defining the HDF5 dataset variables
                 val = '{0}/{1}/{2}'.format(gtx,key,k)
-                h5[gtx][key][k] = fileID.create_dataset(val,np.shape(v),data=v,
-                    dtype=v.dtype, fillvalue=fillvalue, compression='gzip')
-                #-- attach dimensions
-                for dim in ['delta_time']:
-                    h5[gtx][key][k].dims.create_scale(h5[gtx][key][dim], dim)
-                    h5[gtx][key][k].dims[0].attach_scale(h5[gtx][key][dim])
+                if fillvalue:
+                    h5[gtx][key][k] = fileID.create_dataset(val, np.shape(v),
+                        data=v, dtype=v.dtype, fillvalue=fillvalue,
+                        compression='gzip')
+                else:
+                    h5[gtx][key][k] = fileID.create_dataset(val, np.shape(v),
+                        data=v, dtype=v.dtype, compression='gzip')
+                #-- create or attach dimensions for HDF5 variable
+                if DIMENSIONS[gtx][key][k]:
+                    #-- attach dimensions
+                    for i,dim in enumerate(DIMENSIONS[gtx][key][k]):
+                        h5[gtx][key][k].dims[i].attach_scale(h5[gtx][key][dim])
+                else:
+                    #-- make dimension
+                    h5[gtx][key][k].make_scale(k)
                 #-- add HDF5 variable attributes
                 for att_name,att_val in attrs.items():
                     h5[gtx][key][k].attrs[att_name] = att_val
@@ -342,9 +356,9 @@ def HDF5_ATL03_tide_write(IS2_atl03_tide, IS2_atl03_attrs, INPUT=None,
     instrument = 'ATLAS > Advanced Topographic Laser Altimeter System'
     fileID.attrs['instrument'] = instrument
     fileID.attrs['source'] = 'Spacecraft'
-    fileID.attrs['references'] = 'http://nsidc.org/data/icesat2/data.html'
+    fileID.attrs['references'] = 'https://nsidc.org/data/icesat-2'
     fileID.attrs['processing_level'] = '4'
-    #-- add attributes for input ATL03 files
+    #-- add attributes for input ATL03 file
     fileID.attrs['input_files'] = os.path.basename(INPUT)
     #-- find geospatial and temporal ranges
     lnmn,lnmx,ltmn,ltmx,tmn,tmx = (np.inf,-np.inf,np.inf,-np.inf,np.inf,-np.inf)
@@ -377,14 +391,14 @@ def HDF5_ATL03_tide_write(IS2_atl03_tide, IS2_atl03_attrs, INPUT=None,
     #-- convert from seconds since 1980-01-06T00:00:00 to Julian days
     time_julian = 2400000.5 + pyTMD.time.convert_delta_time(gps_seconds - leaps,
         epoch1=(1980,1,6,0,0,0), epoch2=(1858,11,17,0,0,0), scale=1.0/86400.0)
-    #-- convert to calendar date with convert_julian.py
-    YY,MM,DD,HH,MN,SS = convert_julian(time_julian,FORMAT='tuple')
+    #-- convert to calendar date
+    YY,MM,DD,HH,MN,SS = pyTMD.time.convert_julian(time_julian,FORMAT='tuple')
     #-- add attributes with measurement date start, end and duration
-    tcs = datetime.datetime(np.int(YY[0]), np.int(MM[0]), np.int(DD[0]),
-        np.int(HH[0]), np.int(MN[0]), np.int(SS[0]), np.int(1e6*(SS[0] % 1)))
+    tcs = datetime.datetime(int(YY[0]), int(MM[0]), int(DD[0]),
+        int(HH[0]), int(MN[0]), int(SS[0]), int(1e6*(SS[0] % 1)))
     fileID.attrs['time_coverage_start'] = tcs.isoformat()
-    tce = datetime.datetime(np.int(YY[1]), np.int(MM[1]), np.int(DD[1]),
-        np.int(HH[1]), np.int(MN[1]), np.int(SS[1]), np.int(1e6*(SS[1] % 1)))
+    tce = datetime.datetime(int(YY[1]), int(MM[1]), int(DD[1]),
+        int(HH[1]), int(MN[1]), int(SS[1]), int(1e6*(SS[1] % 1)))
     fileID.attrs['time_coverage_end'] = tce.isoformat()
     fileID.attrs['time_coverage_duration'] = '{0:0.0f}'.format(tmx-tmn)
     #-- Closing the HDF5 file
