@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_LPET_icebridge_data.py
-Written by Tyler Sutterley (10/2020)
+Written by Tyler Sutterley (03/2021)
 Calculates long-period equilibrium tidal elevations for correcting Operation
     IceBridge elevation data
 
@@ -28,12 +28,13 @@ PYTHON DEPENDENCIES:
 PROGRAM DEPENDENCIES:
     time.py: utilities for calculating time operations
     utilities: download and management utilities for syncing files
-    convert_julian.py: returns the calendar date and time given a Julian date
     calc_delta_time.py: calculates difference between universal and dynamic time
     compute_equilibrium_tide.py: calculates long-period equilibrium ocean tides
     read_ATM1b_QFIT_binary.py: read ATM1b QFIT binary files (NSIDC version 1)
 
 UPDATE HISTORY:
+    Updated 03/2021: replaced numpy bool/int to prevent deprecation warnings
+    Updated 12/2020: merged time conversion routines into module
     Updated 10/2020: using argparse to set command line parameters
     Updated 09/2020: output days since 1992-01-01 as time variable
     Written 08/2020
@@ -49,7 +50,6 @@ import argparse
 import numpy as np
 import pyTMD.time
 from pyTMD.utilities import get_data_path
-from pyTMD.convert_julian import convert_julian
 from pyTMD.calc_delta_time import calc_delta_time
 from pyTMD.compute_equilibrium_tide import compute_equilibrium_tide
 from read_ATM1b_QFIT_binary.read_ATM1b_QFIT_binary import read_ATM1b_QFIT_binary
@@ -103,22 +103,22 @@ def read_ATM_qfit_file(input_file, input_subsetter):
         #-- number of lines of data within file
         file_lines = file_length(input_file,input_subsetter)
         #-- create output variables with length equal to the number of lines
-        ATM_L1b_input['lat'] = np.zeros_like(file_contents,dtype=np.float)
-        ATM_L1b_input['lon'] = np.zeros_like(file_contents,dtype=np.float)
-        ATM_L1b_input['data'] = np.zeros_like(file_contents,dtype=np.float)
-        hour = np.zeros_like(file_contents,dtype=np.float)
-        minute = np.zeros_like(file_contents,dtype=np.float)
-        second = np.zeros_like(file_contents,dtype=np.float)
+        ATM_L1b_input['lat'] = np.zeros_like(file_contents,dtype=np.float64)
+        ATM_L1b_input['lon'] = np.zeros_like(file_contents,dtype=np.float64)
+        ATM_L1b_input['data'] = np.zeros_like(file_contents,dtype=np.float64)
+        hour = np.zeros_like(file_contents,dtype=np.float64)
+        minute = np.zeros_like(file_contents,dtype=np.float64)
+        second = np.zeros_like(file_contents,dtype=np.float64)
         #-- for each line within the file
         for i,line in enumerate(file_contents):
             #-- find numerical instances within the line
             line_contents = rx.findall(line)
-            ATM_L1b_input['lat'][i] = np.float(line_contents[1])
-            ATM_L1b_input['lon'][i] = np.float(line_contents[2])
-            ATM_L1b_input['data'][i] = np.float(line_contents[3])
-            hour[i] = np.float(line_contents[-1][:2])
-            minute[i] = np.float(line_contents[-1][2:4])
-            second[i] = np.float(line_contents[-1][4:])
+            ATM_L1b_input['lat'][i] = np.float64(line_contents[1])
+            ATM_L1b_input['lon'][i] = np.float64(line_contents[2])
+            ATM_L1b_input['data'][i] = np.float64(line_contents[3])
+            hour[i] = np.float64(line_contents[-1][:2])
+            minute[i] = np.float64(line_contents[-1][2:4])
+            second[i] = np.float64(line_contents[-1][4:])
     #-- Version 1 of ATM QFIT files (binary)
     elif (SFX == 'qi'):
         #-- read input QFIT data file and subset if specified
@@ -130,16 +130,16 @@ def read_ATM_qfit_file(input_file, input_subsetter):
         ATM_L1b_input['data'] = fid['elevation'][:]
         time_hhmmss = fid['time_hhmmss'][:]
         #-- extract hour, minute and second from time_hhmmss
-        hour = np.zeros_like(time_hhmmss,dtype=np.float)
-        minute = np.zeros_like(time_hhmmss,dtype=np.float)
-        second = np.zeros_like(time_hhmmss,dtype=np.float)
+        hour = np.zeros_like(time_hhmmss,dtype=np.float64)
+        minute = np.zeros_like(time_hhmmss,dtype=np.float64)
+        second = np.zeros_like(time_hhmmss,dtype=np.float64)
         #-- for each line within the file
         for i,packed_time in enumerate(time_hhmmss):
             #-- convert to zero-padded string with 3 decimal points
             line_contents = '{0:010.3f}'.format(packed_time)
-            hour[i] = np.float(line_contents[:2])
-            minute[i] = np.float(line_contents[2:4])
-            second[i] = np.float(line_contents[4:])
+            hour[i] = np.float64(line_contents[:2])
+            minute[i] = np.float64(line_contents[2:4])
+            second[i] = np.float64(line_contents[4:])
     #-- Version 2 of ATM QFIT files (HDF5)
     elif (SFX == 'h5'):
         #-- Open the HDF5 file for reading
@@ -152,16 +152,16 @@ def read_ATM_qfit_file(input_file, input_subsetter):
         ATM_L1b_input['data'] = fileID['elevation'][:]
         time_hhmmss = fileID['instrument_parameters']['time_hhmmss'][:]
         #-- extract hour, minute and second from time_hhmmss
-        hour = np.zeros_like(time_hhmmss,dtype=np.float)
-        minute = np.zeros_like(time_hhmmss,dtype=np.float)
-        second = np.zeros_like(time_hhmmss,dtype=np.float)
+        hour = np.zeros_like(time_hhmmss,dtype=np.float64)
+        minute = np.zeros_like(time_hhmmss,dtype=np.float64)
+        second = np.zeros_like(time_hhmmss,dtype=np.float64)
         #-- for each line within the file
         for i,packed_time in enumerate(time_hhmmss):
             #-- convert to zero-padded string with 3 decimal points
             line_contents = '{0:010.3f}'.format(packed_time)
-            hour[i] = np.float(line_contents[:2])
-            minute[i] = np.float(line_contents[2:4])
-            second[i] = np.float(line_contents[4:])
+            hour[i] = np.float64(line_contents[:2])
+            minute[i] = np.float64(line_contents[2:4])
+            second[i] = np.float64(line_contents[4:])
         #-- close the input HDF5 file
         fileID.close()
     #-- calculate the number of leap seconds between GPS time (seconds
@@ -216,14 +216,14 @@ def read_ATM_icessn_file(input_file, input_subsetter):
     ATM_L2_input = {}
     #-- create output variables with length equal to the number of file lines
     for key in file_dtype.keys():
-        ATM_L2_input[key] = np.zeros_like(file_contents, dtype=np.float)
+        ATM_L2_input[key] = np.zeros_like(file_contents, dtype=np.float64)
     #-- for each line within the file
     for line_number,line_entries in enumerate(file_contents):
         #-- find numerical instances within the line
         line_contents = rx.findall(line_entries)
         #-- for each variable of interest: save to dinput as float
         for key,val in file_dtype.items():
-            ATM_L2_input[key][line_number] = np.float(line_contents[val])
+            ATM_L2_input[key][line_number] = np.float64(line_contents[val])
     #-- convert shot time (seconds of day) to J2000
     hour = np.floor(ATM_L2_input['seconds']/3600.0)
     minute = np.floor((ATM_L2_input['seconds'] % 3600)/60.0)
@@ -268,7 +268,7 @@ def read_LVIS_HDF5_file(input_file, input_subsetter):
     regex_pattern = r'{0}_(.*?)(\d+)_(\d+)_(R\d+)_(\d+).H5'.format(mission_flag)
     #-- extract mission, region and other parameters from filename
     MISSION,REGION,YY,MMDD,RLD,SS = re.findall(regex_pattern,input_file).pop()
-    LDS_VERSION = '2.0.2' if (np.int(RLD[1:3]) >= 18) else '1.04'
+    LDS_VERSION = '2.0.2' if (int(RLD[1:3]) >= 18) else '1.04'
     #-- input and output python dictionaries with variables
     file_input = {}
     LVIS_L2_input = {}
@@ -313,10 +313,10 @@ def read_LVIS_HDF5_file(input_file, input_subsetter):
     #-- close the input HDF5 file
     fileID.close()
     #-- output combined variables
-    LVIS_L2_input['data'] = np.zeros_like(file_input['elev'],dtype=np.float)
-    LVIS_L2_input['lon'] = np.zeros_like(file_input['elev'],dtype=np.float)
-    LVIS_L2_input['lat'] = np.zeros_like(file_input['elev'],dtype=np.float)
-    LVIS_L2_input['error'] = np.zeros_like(file_input['elev'],dtype=np.float)
+    LVIS_L2_input['data'] = np.zeros_like(file_input['elev'],dtype=np.float64)
+    LVIS_L2_input['lon'] = np.zeros_like(file_input['elev'],dtype=np.float64)
+    LVIS_L2_input['lat'] = np.zeros_like(file_input['elev'],dtype=np.float64)
+    LVIS_L2_input['error'] = np.zeros_like(file_input['elev'],dtype=np.float64)
     #-- find where elev high is equal to elev low
     #-- see note about using LVIS centroid elevation product
     #-- http://lvis.gsfc.nasa.gov/OIBDataStructure.html
@@ -411,10 +411,10 @@ def compute_LPET_icebridge_data(arg, VERBOSE=False, MODE=0o775):
         #-- early date strings omitted century and millenia (e.g. 93 for 1993)
         if (len(YYMMDD1) == 6):
             ypre,MM1,DD1 = YYMMDD1[:2],YYMMDD1[2:4],YYMMDD1[4:]
-            if (np.float(ypre) >= 90):
-                YY1 = '{0:4.0f}'.format(np.float(ypre) + 1900.0)
+            if (np.float64(ypre) >= 90):
+                YY1 = '{0:4.0f}'.format(np.float64(ypre) + 1900.0)
             else:
-                YY1 = '{0:4.0f}'.format(np.float(ypre) + 2000.0)
+                YY1 = '{0:4.0f}'.format(np.float64(ypre) + 2000.0)
         elif (len(YYMMDD1) == 8):
             YY1,MM1,DD1 = YYMMDD1[:4],YYMMDD1[4:6],YYMMDD1[6:]
     elif OIB in ('LVIS','LVGH'):
@@ -501,7 +501,7 @@ def compute_LPET_icebridge_data(arg, VERBOSE=False, MODE=0o775):
     fid.attrs['project'] = 'NASA_Operation_IceBridge'
     fid.attrs['processing_level'] = '4'
     fid.attrs['date_created'] = time.strftime('%Y-%m-%d',time.localtime())
-    #-- add attributes for input files
+    #-- add attributes for input file
     fid.attrs['elevation_file'] = os.path.basename(input_file)
     #-- add geospatial and temporal attributes
     fid.attrs['geospatial_lat_min'] = dinput['lat'].min()
@@ -516,8 +516,8 @@ def compute_LPET_icebridge_data(arg, VERBOSE=False, MODE=0o775):
     time_range = np.array([np.min(tide_time),np.max(tide_time)])
     time_julian = 2400000.5 + pyTMD.time.convert_delta_time(time_range,
         epoch1=(1992,1,1,0,0,0), epoch2=(1858,11,17,0,0,0), scale=1.0)
-    #-- convert to calendar date with convert_julian.py
-    cal = convert_julian(time_julian,ASTYPE=np.int)
+    #-- convert to calendar date
+    cal = pyTMD.time.convert_julian(time_julian,ASTYPE=int)
     #-- add attributes with measurement date start, end and duration
     args = (cal['hour'][0],cal['minute'][0],cal['second'][0])
     fid.attrs['RangeBeginningTime'] = '{0:02d}:{1:02d}:{2:02d}'.format(*args)
