@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_tidal_elevations.py
-Written by Tyler Sutterley (11/2020)
+Written by Tyler Sutterley (03/2021)
 Calculates tidal elevations for an input file
 
 Uses OTIS format tidal solutions provided by Ohio State University and ESR
@@ -26,6 +26,7 @@ COMMAND LINE OPTIONS:
         TPXO9-atlas
         TPXO9-atlas-v2
         TPXO9-atlas-v3
+        TPXO9-atlas-v4
         TPXO9.1
         TPXO8-atlas
         TPXO7.2
@@ -53,7 +54,7 @@ COMMAND LINE OPTIONS:
     -t X, --type X: input data type
         drift: drift buoys or satellite/airborne altimetry (time per data point)
         grid: spatial grids or images (single time for all data points)
-    --epoch X: Reference epoch of input time (default Modified Julian Day)
+    -e X, --epoch X: Reference epoch of input time (default Modified Julian Day)
         days since 1858-11-17T00:00:00
     -d X, --deltatime X: Input delta time for files without date information
         can be set to 0 to use exact calendar date from epoch
@@ -64,6 +65,7 @@ COMMAND LINE OPTIONS:
         linear
         nearest
         bilinear
+    -E X, --extrapolate X: Extrapolate with nearest-neighbors
     -V, --verbose: Verbose output of processing run
     -M X, --mode X: Permission mode of output file
 
@@ -98,9 +100,15 @@ PROGRAM DEPENDENCIES:
     read_netcdf_model.py: extract tidal harmonic constants from netcdf models
     read_GOT_model.py: extract tidal harmonic constants from GSFC GOT models
     read_FES_model.py: extract tidal harmonic constants from FES tide models
+    bilinear_interp.py: bilinear interpolation of data to coordinates
+    nearest_extrap.py: nearest-neighbor extrapolation of data to coordinates
     predict_tide_drift.py: predict tidal elevations using harmonic constants
 
 UPDATE HISTORY:
+    Updated 03/2021: added TPXO9-atlas-v4 in binary OTIS format
+        simplified netcdf inputs to be similar to binary OTIS read program
+    Updated 02/2021: replaced numpy bool to prevent deprecation warning
+    Updated 12/2020: added valid data extrapolation with nearest_extrap
     Updated 11/2020: added model constituents from TPXO9-atlas-v3
         added options to read from and write to geotiff image files
     Updated 10/2020: using argparse to set command line parameters
@@ -139,7 +147,8 @@ from pyTMD.read_FES_model import extract_FES_constants
 def compute_tidal_elevations(tide_dir, input_file, output_file,
     TIDE_MODEL=None, FORMAT='csv', VARIABLES=['time','lat','lon','data'],
     HEADER=0, TYPE='drift', TIME_UNITS='days since 1858-11-17T00:00:00',
-    TIME=None, PROJECTION='4326', METHOD='spline', VERBOSE=False, MODE=0o775):
+    TIME=None, PROJECTION='4326', METHOD='spline', EXTRAPOLATE=False,
+    VERBOSE=False, MODE=0o775):
 
     #-- select between tide models
     if (TIDE_MODEL == 'CATS0201'):
@@ -173,37 +182,41 @@ def compute_tidal_elevations(tide_dir, input_file, output_file,
         model_type = 'z'
     elif (TIDE_MODEL == 'TPXO9-atlas'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas')
-        grid_file = 'grid_tpxo9_atlas.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30.nc.gz','h_o1_tpxo9_atlas_30.nc.gz',
             'h_p1_tpxo9_atlas_30.nc.gz','h_k1_tpxo9_atlas_30.nc.gz',
             'h_n2_tpxo9_atlas_30.nc.gz','h_m2_tpxo9_atlas_30.nc.gz',
             'h_s2_tpxo9_atlas_30.nc.gz','h_k2_tpxo9_atlas_30.nc.gz',
             'h_m4_tpxo9_atlas_30.nc.gz','h_ms4_tpxo9_atlas_30.nc.gz',
             'h_mn4_tpxo9_atlas_30.nc.gz','h_2n2_tpxo9_atlas_30.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = 'http://volkov.oce.orst.edu/tides/tpxo9_atlas.html'
         output_variable = 'tide_ocean'
         variable_long_name = 'Ocean_Tide'
         model_format = 'netcdf'
         model_type = 'z'
         model_scale = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'TPXO9-atlas-v2'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas_v2')
-        grid_file = 'grid_tpxo9_atlas_30_v2.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v2.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30_v2.nc.gz','h_o1_tpxo9_atlas_30_v2.nc.gz',
             'h_p1_tpxo9_atlas_30_v2.nc.gz','h_k1_tpxo9_atlas_30_v2.nc.gz',
             'h_n2_tpxo9_atlas_30_v2.nc.gz','h_m2_tpxo9_atlas_30_v2.nc.gz',
             'h_s2_tpxo9_atlas_30_v2.nc.gz','h_k2_tpxo9_atlas_30_v2.nc.gz',
             'h_m4_tpxo9_atlas_30_v2.nc.gz','h_ms4_tpxo9_atlas_30_v2.nc.gz',
             'h_mn4_tpxo9_atlas_30_v2.nc.gz','h_2n2_tpxo9_atlas_30_v2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = 'https://www.tpxo.net/global/tpxo9-atlas'
         output_variable = 'tide_ocean'
         variable_long_name = 'Ocean_Tide'
         model_format = 'netcdf'
         model_type = 'z'
         model_scale = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'TPXO9-atlas-v3'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas_v3')
-        grid_file = 'grid_tpxo9_atlas_30_v3.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v3.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30_v3.nc.gz','h_o1_tpxo9_atlas_30_v3.nc.gz',
             'h_p1_tpxo9_atlas_30_v3.nc.gz','h_k1_tpxo9_atlas_30_v3.nc.gz',
             'h_n2_tpxo9_atlas_30_v3.nc.gz','h_m2_tpxo9_atlas_30_v3.nc.gz',
@@ -211,12 +224,31 @@ def compute_tidal_elevations(tide_dir, input_file, output_file,
             'h_m4_tpxo9_atlas_30_v3.nc.gz','h_ms4_tpxo9_atlas_30_v3.nc.gz',
             'h_mn4_tpxo9_atlas_30_v3.nc.gz','h_2n2_tpxo9_atlas_30_v3.nc.gz',
             'h_mf_tpxo9_atlas_30_v3.nc.gz','h_mm_tpxo9_atlas_30_v3.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = 'https://www.tpxo.net/global/tpxo9-atlas'
         variable = 'tide_ocean'
         long_name = "Ocean_Tide"
         model_format = 'netcdf'
         model_type = 'z'
         model_scale = 1.0/1000.0
+        GZIP = True
+    elif (TIDE_MODEL == 'TPXO9-atlas-v4'):
+        model_directory = os.path.join(tide_dir,'TPXO9_atlas_v4')
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v4')
+        model_files = ['h_q1_tpxo9_atlas_30_v4','h_o1_tpxo9_atlas_30_v4',
+            'h_p1_tpxo9_atlas_30_v4','h_k1_tpxo9_atlas_30_v4',
+            'h_n2_tpxo9_atlas_30_v4','h_m2_tpxo9_atlas_30_v4',
+            'h_s2_tpxo9_atlas_30_v4','h_k2_tpxo9_atlas_30_v4',
+            'h_m4_tpxo9_atlas_30_v4','h_ms4_tpxo9_atlas_30_v4',
+            'h_mn4_tpxo9_atlas_30_v4','h_2n2_tpxo9_atlas_30_v4',
+            'h_mf_tpxo9_atlas_30_v4','h_mm_tpxo9_atlas_30_v4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
+        reference = 'https://www.tpxo.net/global/tpxo9-atlas'
+        variable = 'tide_ocean'
+        long_name = "Ocean_Tide"
+        model_format = 'OTIS'
+        EPSG = '4326'
+        model_type = 'z'
     elif (TIDE_MODEL == 'TPXO9.1'):
         grid_file = os.path.join(tide_dir,'TPXO9.1','DATA','grid_tpxo9')
         model_file = os.path.join(tide_dir,'TPXO9.1','DATA','h_tpxo9.v1')
@@ -287,71 +319,77 @@ def compute_tidal_elevations(tide_dir, input_file, output_file,
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         output_variable = 'tide_ocean'
         variable_long_name = 'Ocean_Tide'
         model_format = 'GOT'
         model_scale = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.7_load'):
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         output_variable = 'tide_load'
         variable_long_name = 'Load_Tide'
         model_format = 'GOT'
         model_scale = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.8'):
         model_directory = os.path.join(tide_dir,'got4.8','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         output_variable = 'tide_ocean'
         variable_long_name = 'Ocean_Tide'
         model_format = 'GOT'
         model_scale = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.8_load'):
         model_directory = os.path.join(tide_dir,'got4.8','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         output_variable = 'tide_load'
         variable_long_name = 'Load_Tide'
         model_format = 'GOT'
         model_scale = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.10'):
         model_directory = os.path.join(tide_dir,'GOT4.10c','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         output_variable = 'tide_ocean'
         variable_long_name = 'Ocean_Tide'
         model_format = 'GOT'
         model_scale = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.10_load'):
         model_directory = os.path.join(tide_dir,'GOT4.10c','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         output_variable = 'tide_load'
         variable_long_name = 'Load_Tide'
         model_format = 'GOT'
         model_scale = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'FES2014'):
         model_directory = os.path.join(tide_dir,'fes2014','ocean_tide')
         model_files = ['2n2.nc.gz','eps2.nc.gz','j1.nc.gz','k1.nc.gz',
@@ -361,16 +399,18 @@ def compute_tidal_elevations(tide_dir, input_file, output_file,
             'mu2.nc.gz','n2.nc.gz','n4.nc.gz','nu2.nc.gz','o1.nc.gz','p1.nc.gz',
             'q1.nc.gz','r2.nc.gz','s1.nc.gz','s2.nc.gz','s4.nc.gz','sa.nc.gz',
             'ssa.nc.gz','t2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['2n2','eps2','j1','k1','k2','l2','lambda2','m2','m3','m4','m6',
             'm8','mf','mks2','mm','mn4','ms4','msf','msqm','mtm','mu2','n2',
             'n4','nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
-        reference = ('https://www.aviso.altimetry.fr/data/products/'
+        reference = ('https://www.aviso.altimetry.fr/en/data/products'
             'auxiliary-products/global-tide-fes.html')
         output_variable = 'tide_ocean'
         variable_long_name = 'Ocean_Tide'
         model_format = 'FES'
         model_type = 'z'
         model_scale = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'FES2014_load'):
         model_directory = os.path.join(tide_dir,'fes2014','load_tide')
         model_files = ['2n2.nc.gz','eps2.nc.gz','j1.nc.gz','k1.nc.gz',
@@ -380,16 +420,18 @@ def compute_tidal_elevations(tide_dir, input_file, output_file,
             'mu2.nc.gz','n2.nc.gz','n4.nc.gz','nu2.nc.gz','o1.nc.gz','p1.nc.gz',
             'q1.nc.gz','r2.nc.gz','s1.nc.gz','s2.nc.gz','s4.nc.gz','sa.nc.gz',
             'ssa.nc.gz','t2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['2n2','eps2','j1','k1','k2','l2','lambda2','m2','m3','m4','m6',
             'm8','mf','mks2','mm','mn4','ms4','msf','msqm','mtm','mu2','n2',
             'n4','nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
-        reference = ('https://www.aviso.altimetry.fr/data/products/'
+        reference = ('https://www.aviso.altimetry.fr/en/data/products'
             'auxiliary-products/global-tide-fes.html')
         output_variable = 'tide_load'
         variable_long_name = 'Load_Tide'
         model_format = 'FES'
         model_type = 'z'
         model_scale = 1.0/100.0
+        GZIP = True
 
     #-- invalid value
     fill_value = -9999.0
@@ -466,31 +508,31 @@ def compute_tidal_elevations(tide_dir, input_file, output_file,
         epoch1=epoch1, epoch2=(1992,1,1,0,0,0), scale=1.0/86400.0)
     #-- number of time points
     nt = len(tide_time)
+    #-- delta time (TT - UT1) file
+    delta_file = get_data_path(['data','merged_deltat.data'])
 
     #-- read tidal constants and interpolate to grid points
     if model_format in ('OTIS','ATLAS'):
         amp,ph,D,c = extract_tidal_constants(lon.flatten(), lat.flatten(),
-            grid_file, model_file, EPSG, TYPE=model_type, METHOD=METHOD)
+            grid_file, model_file, EPSG, TYPE=model_type, METHOD=METHOD,
+            EXTRAPOLATE=EXTRAPOLATE)
         deltat = np.zeros((nt))
     elif (model_format == 'netcdf'):
         amp,ph,D,c = extract_netcdf_constants(lon.flatten(), lat.flatten(),
-            model_directory, grid_file, model_files, TYPE=model_type,
-            METHOD=METHOD, SCALE=model_scale)
+            grid_file, model_file, TYPE=model_type, METHOD=METHOD,
+            EXTRAPOLATE=EXTRAPOLATE, SCALE=model_scale, GZIP=GZIP)
         deltat = np.zeros((nt))
     elif (model_format == 'GOT'):
-        amp,ph = extract_GOT_constants(lon.flatten(), lat.flatten(),
-            model_directory, model_files, METHOD=METHOD, SCALE=model_scale)
-        #-- convert times from modified julian days to days since 1992-01-01
+        amp,ph,c = extract_GOT_constants(lon.flatten(), lat.flatten(),
+            model_file, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
+            SCALE=model_scale, GZIP=GZIP)
         #-- interpolate delta times from calendar dates to tide time
-        delta_file = get_data_path(['data','merged_deltat.data'])
         deltat = calc_delta_time(delta_file,tide_time)
     elif (model_format == 'FES'):
         amp,ph = extract_FES_constants(lon.flatten(), lat.flatten(),
-            model_directory, model_files, TYPE=model_type, VERSION=TIDE_MODEL,
-            METHOD=METHOD, SCALE=model_scale)
-        #-- convert times from modified julian days to days since 1992-01-01
+            model_file, TYPE=model_type, VERSION=TIDE_MODEL, METHOD=METHOD,
+            EXTRAPOLATE=EXTRAPOLATE, SCALE=model_scale, GZIP=GZIP)
         #-- interpolate delta times from calendar dates to tide time
-        delta_file = get_data_path(['data','merged_deltat.data'])
         deltat = calc_delta_time(delta_file,tide_time)
 
     #-- calculate complex phase in radians for Euler's
@@ -501,7 +543,7 @@ def compute_tidal_elevations(tide_dir, input_file, output_file,
     #-- predict tidal elevations at time and infer minor corrections
     if (TYPE == 'grid'):
         tide = np.ma.zeros((ny,nx,nt),fill_value=fill_value)
-        tide.mask = np.zeros((ny,nx,nt),dtype=np.bool)
+        tide.mask = np.zeros((ny,nx,nt),dtype=bool)
         for i in range(nt):
             TIDE = predict_tide(tide_time[i], hc, c,
                 DELTAT=deltat[i], CORRECTIONS=model_format)
@@ -558,7 +600,7 @@ def main():
         help='Working data directory')
     #-- tide model to use
     model_choices = ('CATS0201','CATS2008','CATS2008_load',
-        'TPXO9-atlas','TPXO9-atlas-v2','TPXO9-atlas-v3',
+        'TPXO9-atlas','TPXO9-atlas-v2','TPXO9-atlas-v3','TPXO9-atlas-v4',
         'TPXO9.1','TPXO8-atlas','TPXO7.2','TPXO7.2_load',
         'AODTM-5','AOTIM-5','AOTIM-5-2018',
         'GOT4.7','GOT4.7_load','GOT4.8','GOT4.8_load','GOT4.10','GOT4.10_load',
@@ -588,7 +630,7 @@ def main():
         help='Input data type')
     #-- time epoch (default Modified Julian Days)
     #-- in form "time-units since yyyy-mm-dd hh:mm:ss"
-    parser.add_argument('--epoch','-E',
+    parser.add_argument('--epoch','-e',
         type=str, default='days since 1858-11-17T00:00:00',
         help='Reference epoch of input time')
     #-- input delta time for files without date information
@@ -604,6 +646,10 @@ def main():
         metavar='METHOD', type=str, default='spline',
         choices=('spline','linear','nearest','bilinear'),
         help='Spatial interpolation method')
+    #-- extrapolate with nearest-neighbors
+    parser.add_argument('--extrapolate','-E',
+        default=False, action='store_true',
+        help='Extrapolate with nearest-neighbors')
     #-- verbose output of processing run
     #-- print information about each input and output file
     parser.add_argument('--verbose','-V',
@@ -626,7 +672,8 @@ def main():
         FORMAT=args.format, TIDE_MODEL=args.tide, VARIABLES=args.variables,
         HEADER=args.header, TYPE=args.type, TIME_UNITS=args.epoch,
         TIME=args.deltatime, PROJECTION=args.projection,
-        METHOD=args.interpolate, VERBOSE=args.verbose, MODE=args.mode)
+        METHOD=args.interpolate, EXTRAPOLATE=args.extrapolate,
+        VERBOSE=args.verbose, MODE=args.mode)
 
 #-- run main program
 if __name__ == '__main__':
