@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_tides_ICESat2_ATL03.py
-Written by Tyler Sutterley (11/2020)
+Written by Tyler Sutterley (04/2021)
 Calculates tidal elevations for correcting ICESat-2 photon height data
 Calculated at ATL03 segment level using reference photon geolocation and time
 Segment level corrections can be applied to the individual photon events (PEs)
@@ -22,6 +22,7 @@ COMMAND LINE OPTIONS:
         TPXO9-atlas
         TPXO9-atlas-v2
         TPXO9-atlas-v3
+        TPXO9-atlas-v4
         TPXO9.1
         TPXO8-atlas
         TPXO7.2
@@ -42,6 +43,7 @@ COMMAND LINE OPTIONS:
         linear
         nearest
         bilinear
+    -E X, --extrapolate X: Extrapolate with nearest-neighbors
     -M X, --mode X: Permission mode of directories and files created
     -V, --verbose: Output information about each created file
 
@@ -60,20 +62,28 @@ PROGRAM DEPENDENCIES:
     read_ICESat2_ATL03.py: reads ICESat-2 global geolocated photon data files
     time.py: utilities for calculating time operations
     utilities: download and management utilities for syncing files
-    convert_julian.py: returns the calendar date and time given a Julian date
     calc_astrol_longitudes.py: computes the basic astronomical mean longitudes
     calc_delta_time.py: calculates difference between universal and dynamic time
     convert_ll_xy.py: convert lat/lon points to and from projected coordinates
     infer_minor_corrections.py: return corrections for minor constituents
     load_constituent.py: loads parameters for a given tidal constituent
     load_nodal_corrections.py: load the nodal corrections for tidal constituents
-    predict_tide_drift.py: predict tidal elevations using harmonic constants
     read_tide_model.py: extract tidal harmonic constants from OTIS tide models
     read_netcdf_model.py: extract tidal harmonic constants from netcdf models
     read_GOT_model.py: extract tidal harmonic constants from GSFC GOT models
     read_FES_model.py: extract tidal harmonic constants from FES tide models
+    bilinear_interp.py: bilinear interpolation of data to coordinates
+    nearest_extrap.py: nearest-neighbor extrapolation of data to coordinates
+    predict_tide_drift.py: predict tidal elevations using harmonic constants
 
 UPDATE HISTORY:
+    Updated 04/2021: can use a generically named ATL03 file as input
+    Updated 03/2021: added TPXO9-atlas-v4 in binary OTIS format
+        simplified netcdf inputs to be similar to binary OTIS read program
+        replaced numpy bool/int to prevent deprecation warnings
+    Updated 12/2020: H5py deprecation warning change to use make_scale
+        added valid data extrapolation with nearest_extrap
+        merged time conversion routines into module
     Updated 11/2020: added model constituents from TPXO9-atlas-v3
     Updated 10/2020: using argparse to set command line parameters
     Updated 08/2020: using builtin time operations.  python3 regular expressions
@@ -101,7 +111,6 @@ import datetime
 import numpy as np
 import pyTMD.time
 import pyTMD.utilities
-from pyTMD.convert_julian import convert_julian
 from pyTMD.calc_delta_time import calc_delta_time
 from pyTMD.read_tide_model import extract_tidal_constants
 from pyTMD.read_netcdf_model import extract_netcdf_constants
@@ -114,8 +123,8 @@ from icesat2_toolkit.read_ICESat2_ATL03 import read_HDF5_ATL03_main, \
 
 #-- PURPOSE: read ICESat-2 geolocated photon data (ATL03) from NSIDC
 #-- compute tides at points and times using tidal model driver algorithms
-def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
-    VERBOSE=False, MODE=0o775):
+def compute_tides_ICESat2(tide_dir, INPUT_FILE, TIDE_MODEL=None, METHOD='spline',
+    EXTRAPOLATE=False, VERBOSE=False, MODE=0o775):
     #-- select between tide models
     if (TIDE_MODEL == 'CATS0201'):
         grid_file = os.path.join(tide_dir,'cats0201_tmd','grid_CATS')
@@ -155,13 +164,14 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         TYPE = 'z'
     elif (TIDE_MODEL == 'TPXO9-atlas'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas')
-        grid_file = 'grid_tpxo9_atlas.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30.nc.gz','h_o1_tpxo9_atlas_30.nc.gz',
             'h_p1_tpxo9_atlas_30.nc.gz','h_k1_tpxo9_atlas_30.nc.gz',
             'h_n2_tpxo9_atlas_30.nc.gz','h_m2_tpxo9_atlas_30.nc.gz',
             'h_s2_tpxo9_atlas_30.nc.gz','h_k2_tpxo9_atlas_30.nc.gz',
             'h_m4_tpxo9_atlas_30.nc.gz','h_ms4_tpxo9_atlas_30.nc.gz',
             'h_mn4_tpxo9_atlas_30.nc.gz','h_2n2_tpxo9_atlas_30.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = 'http://volkov.oce.orst.edu/tides/tpxo9_atlas.html'
         variable = 'tide_ocean'
         long_name = "Ocean Tide"
@@ -171,15 +181,17 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         model_format = 'netcdf'
         TYPE = 'z'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'TPXO9-atlas-v2'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas_v2')
-        grid_file = 'grid_tpxo9_atlas_30_v2.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v2.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30_v2.nc.gz','h_o1_tpxo9_atlas_30_v2.nc.gz',
             'h_p1_tpxo9_atlas_30_v2.nc.gz','h_k1_tpxo9_atlas_30_v2.nc.gz',
             'h_n2_tpxo9_atlas_30_v2.nc.gz','h_m2_tpxo9_atlas_30_v2.nc.gz',
             'h_s2_tpxo9_atlas_30_v2.nc.gz','h_k2_tpxo9_atlas_30_v2.nc.gz',
             'h_m4_tpxo9_atlas_30_v2.nc.gz','h_ms4_tpxo9_atlas_30_v2.nc.gz',
             'h_mn4_tpxo9_atlas_30_v2.nc.gz','h_2n2_tpxo9_atlas_30_v2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = 'https://www.tpxo.net/global/tpxo9-atlas'
         variable = 'tide_ocean'
         long_name = "Ocean Tide"
@@ -189,9 +201,10 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         model_format = 'netcdf'
         TYPE = 'z'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'TPXO9-atlas-v3'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas_v3')
-        grid_file = 'grid_tpxo9_atlas_30_v3.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v3.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30_v3.nc.gz','h_o1_tpxo9_atlas_30_v3.nc.gz',
             'h_p1_tpxo9_atlas_30_v3.nc.gz','h_k1_tpxo9_atlas_30_v3.nc.gz',
             'h_n2_tpxo9_atlas_30_v3.nc.gz','h_m2_tpxo9_atlas_30_v3.nc.gz',
@@ -199,6 +212,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             'h_m4_tpxo9_atlas_30_v3.nc.gz','h_ms4_tpxo9_atlas_30_v3.nc.gz',
             'h_mn4_tpxo9_atlas_30_v3.nc.gz','h_2n2_tpxo9_atlas_30_v3.nc.gz',
             'h_mf_tpxo9_atlas_30_v3.nc.gz','h_mm_tpxo9_atlas_30_v3.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = 'https://www.tpxo.net/global/tpxo9-atlas'
         variable = 'tide_ocean'
         long_name = "Ocean Tide"
@@ -208,6 +222,27 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         model_format = 'netcdf'
         TYPE = 'z'
         SCALE = 1.0/1000.0
+        GZIP = True
+    elif (TIDE_MODEL == 'TPXO9-atlas-v4'):
+        model_directory = os.path.join(tide_dir,'TPXO9_atlas_v4')
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v4')
+        model_files = ['h_q1_tpxo9_atlas_30_v4','h_o1_tpxo9_atlas_30_v4',
+            'h_p1_tpxo9_atlas_30_v4','h_k1_tpxo9_atlas_30_v4',
+            'h_n2_tpxo9_atlas_30_v4','h_m2_tpxo9_atlas_30_v4',
+            'h_s2_tpxo9_atlas_30_v4','h_k2_tpxo9_atlas_30_v4',
+            'h_m4_tpxo9_atlas_30_v4','h_ms4_tpxo9_atlas_30_v4',
+            'h_mn4_tpxo9_atlas_30_v4','h_2n2_tpxo9_atlas_30_v4',
+            'h_mf_tpxo9_atlas_30_v4','h_mm_tpxo9_atlas_30_v4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
+        reference = 'https://www.tpxo.net/global/tpxo9-atlas'
+        variable = 'tide_ocean'
+        long_name = "Ocean Tide"
+        description = ("Ocean Tides including diurnal and semi-diurnal "
+            "(harmonic analysis), and longer period tides (dynamic and "
+            "self-consistent equilibrium).")
+        model_format = 'OTIS'
+        EPSG = '4326'
+        TYPE = 'z'
     elif (TIDE_MODEL == 'TPXO9.1'):
         grid_file = os.path.join(tide_dir,'TPXO9.1','DATA','grid_tpxo9')
         model_file = os.path.join(tide_dir,'TPXO9.1','DATA','h_tpxo9.v1')
@@ -297,6 +332,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
@@ -307,11 +343,13 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             "self-consistent equilibrium).")
         model_format = 'GOT'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.7_load'):
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
@@ -320,10 +358,12 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         description = "Local displacement due to Ocean Loading (-6 to 0 cm)"
         model_format = 'GOT'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.8'):
         model_directory = os.path.join(tide_dir,'got4.8','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
@@ -334,11 +374,13 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             "self-consistent equilibrium).")
         model_format = 'GOT'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.8_load'):
         model_directory = os.path.join(tide_dir,'got4.8','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
@@ -347,10 +389,12 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         description = "Local displacement due to Ocean Loading (-6 to 0 cm)"
         model_format = 'GOT'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.10'):
         model_directory = os.path.join(tide_dir,'GOT4.10c','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
@@ -361,11 +405,13 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             "self-consistent equilibrium).")
         model_format = 'GOT'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.10_load'):
         model_directory = os.path.join(tide_dir,'GOT4.10c','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
@@ -374,6 +420,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         description = "Local displacement due to Ocean Loading (-6 to 0 cm)"
         model_format = 'GOT'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'FES2014'):
         model_directory = os.path.join(tide_dir,'fes2014','ocean_tide')
         model_files = ['2n2.nc.gz','eps2.nc.gz','j1.nc.gz','k1.nc.gz',
@@ -383,10 +430,11 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             'mu2.nc.gz','n2.nc.gz','n4.nc.gz','nu2.nc.gz','o1.nc.gz','p1.nc.gz',
             'q1.nc.gz','r2.nc.gz','s1.nc.gz','s2.nc.gz','s4.nc.gz','sa.nc.gz',
             'ssa.nc.gz','t2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['2n2','eps2','j1','k1','k2','l2','lambda2','m2','m3','m4','m6',
             'm8','mf','mks2','mm','mn4','ms4','msf','msqm','mtm','mu2','n2',
             'n4','nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
-        reference = ('https://www.aviso.altimetry.fr/data/products/'
+        reference = ('https://www.aviso.altimetry.fr/en/data/products'
             'auxiliary-products/global-tide-fes.html')
         variable = 'tide_ocean'
         long_name = "Ocean Tide"
@@ -396,6 +444,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         model_format = 'FES'
         TYPE = 'z'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'FES2014_load'):
         model_directory = os.path.join(tide_dir,'fes2014','load_tide')
         model_files = ['2n2.nc.gz','eps2.nc.gz','j1.nc.gz','k1.nc.gz',
@@ -405,10 +454,11 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
             'mu2.nc.gz','n2.nc.gz','n4.nc.gz','nu2.nc.gz','o1.nc.gz','p1.nc.gz',
             'q1.nc.gz','r2.nc.gz','s1.nc.gz','s2.nc.gz','s4.nc.gz','sa.nc.gz',
             'ssa.nc.gz','t2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['2n2','eps2','j1','k1','k2','l2','lambda2','m2','m3','m4','m6',
             'm8','mf','mks2','mm','mn4','ms4','msf','msqm','mtm','mu2','n2',
             'n4','nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
-        reference = ('https://www.aviso.altimetry.fr/data/products/'
+        reference = ('https://www.aviso.altimetry.fr/en/data/products'
             'auxiliary-products/global-tide-fes.html')
         variable = 'tide_load'
         long_name = "Load Tide"
@@ -416,24 +466,39 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         model_format = 'FES'
         TYPE = 'z'
         SCALE = 1.0/100.0
+        GZIP = True
 
-    #-- read data from FILE
-    print('{0} -->'.format(os.path.basename(FILE))) if VERBOSE else None
-    IS2_atl03_mds,IS2_atl03_attrs,IS2_atl03_beams = read_HDF5_ATL03_main(FILE,
+    #-- read data from input file
+    print('{0} -->'.format(os.path.basename(INPUT_FILE))) if VERBOSE else None
+    IS2_atl03_mds,IS2_atl03_attrs,IS2_atl03_beams = read_HDF5_ATL03_main(INPUT_FILE,
         ATTRIBUTES=True)
-    DIRECTORY = os.path.dirname(FILE)
+    DIRECTORY = os.path.dirname(INPUT_FILE)
     #-- extract parameters from ICESat-2 ATLAS HDF5 file name
     rx = re.compile(r'(processed_)?(ATL\d{2})_(\d{4})(\d{2})(\d{2})(\d{2})'
-        '(\d{2})(\d{2})_(\d{4})(\d{2})(\d{2})_(\d{3})_(\d{2})(.*?).h5$')
-    SUB,PRD,YY,MM,DD,HH,MN,SS,TRK,CYCL,GRAN,RL,VERS,AUX = rx.findall(FILE).pop()
+        r'(\d{2})(\d{2})_(\d{4})(\d{2})(\d{2})_(\d{3})_(\d{2})(.*?).h5$')
+    try:
+        SUB,PRD,YY,MM,DD,HH,MN,SS,TRK,CYCL,GRAN,RL,VERS,AUX = rx.findall(INPUT_FILE).pop()
+    except:
+        #-- output tide HDF5 file (generic)
+        fileBasename,fileExtension = os.path.splitext(INPUT_FILE)
+        args = (fileBasename,TIDE_MODEL,fileExtension)
+        OUTPUT_FILE = '{0}_{1}_TIDES{2}'.format(*args)
+    else:
+        #-- output tide HDF5 file for ASAS/NSIDC granules
+        args = (PRD,TIDE_MODEL,YY,MM,DD,HH,MN,SS,TRK,CYCL,GRAN,RL,VERS,AUX)
+        file_format = '{0}_{1}_TIDES_{2}{3}{4}{5}{6}{7}_{8}{9}{10}_{11}_{12}{13}.h5'
+        OUTPUT_FILE = file_format.format(*args)
 
     #-- number of GPS seconds between the GPS epoch
     #-- and ATLAS Standard Data Product (SDP) epoch
     atlas_sdp_gps_epoch = IS2_atl03_mds['ancillary_data']['atlas_sdp_gps_epoch']
+    #-- delta time (TT - UT1) file
+    delta_file = pyTMD.utilities.get_data_path(['data','merged_deltat.data'])
 
     #-- copy variables for outputting to HDF5 file
     IS2_atl03_tide = {}
     IS2_atl03_fill = {}
+    IS2_atl03_dims = {}
     IS2_atl03_tide_attrs = {}
     #-- number of GPS seconds between the GPS epoch (1980-01-06T00:00:00Z UTC)
     #-- and ATLAS Standard Data Product (SDP) epoch (2018-01-01T00:00:00Z UTC)
@@ -453,10 +518,11 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- output data dictionaries for beam
         IS2_atl03_tide[gtx] = dict(geolocation={}, geophys_corr={})
         IS2_atl03_fill[gtx] = dict(geolocation={}, geophys_corr={})
+        IS2_atl03_dims[gtx] = dict(geolocation={}, geophys_corr={})
         IS2_atl03_tide_attrs[gtx] = dict(geolocation={}, geophys_corr={})
 
         #-- read data and attributes for beam
-        val,attrs = read_HDF5_ATL03_beam(FILE,gtx,ATTRIBUTES=True)
+        val,attrs = read_HDF5_ATL03_beam(INPUT_FILE,gtx,ATTRIBUTES=True)
         #-- number of segments
         n_seg = len(val['geolocation']['segment_id'])
         #-- extract variables for computing tides
@@ -475,23 +541,25 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- read tidal constants and interpolate to grid points
         if model_format in ('OTIS','ATLAS'):
             amp,ph,D,c = extract_tidal_constants(lon, lat, grid_file,
-                model_file, EPSG, TYPE=TYPE, METHOD=METHOD, GRID=model_format)
+                model_file, EPSG, TYPE=TYPE, METHOD=METHOD,
+                EXTRAPOLATE=EXTRAPOLATE, GRID=model_format)
             deltat = np.zeros_like(tide_time)
         elif (model_format == 'netcdf'):
-            amp,ph,D,c = extract_netcdf_constants(lon, lat, model_directory,
-                grid_file, model_files, TYPE=TYPE, METHOD=METHOD, SCALE=SCALE)
+            amp,ph,D,c = extract_netcdf_constants(lon, lat, grid_file,
+                model_file, TYPE=TYPE, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
+                SCALE=SCALE, GZIP=GZIP)
             deltat = np.zeros_like(tide_time)
         elif (model_format == 'GOT'):
-            amp,ph = extract_GOT_constants(lon, lat, model_directory,
-                model_files, METHOD=METHOD, SCALE=SCALE)
+            amp,ph,c = extract_GOT_constants(lon, lat, model_file,
+                METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE,
+                GZIP=GZIP)
             #-- interpolate delta times from calendar dates to tide time
-            delta_file = pyTMD.utilities.get_data_path(['data','merged_deltat.data'])
             deltat = calc_delta_time(delta_file, tide_time)
         elif (model_format == 'FES'):
-            amp,ph = extract_FES_constants(lon,lat,model_directory,model_files,
-                TYPE=TYPE, VERSION=TIDE_MODEL, METHOD=METHOD, SCALE=SCALE)
+            amp,ph = extract_FES_constants(lon, lat, model_file, TYPE=TYPE,
+                VERSION=TIDE_MODEL, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
+                SCALE=SCALE, GZIP=GZIP)
             #-- interpolate delta times from calendar dates to tide time
-            delta_file = pyTMD.utilities.get_data_path(['data','merged_deltat.data'])
             deltat = calc_delta_time(delta_file, tide_time)
 
         #-- calculate complex phase in radians for Euler's
@@ -539,6 +607,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- delta time in geolocation group
         IS2_atl03_tide[gtx]['geolocation']['delta_time'] = delta_time
         IS2_atl03_fill[gtx]['geolocation']['delta_time'] = None
+        IS2_atl03_dims[gtx]['geolocation']['delta_time'] = None
         IS2_atl03_tide_attrs[gtx]['geolocation']['delta_time'] = {}
         IS2_atl03_tide_attrs[gtx]['geolocation']['delta_time']['units'] = "seconds since 2018-01-01"
         IS2_atl03_tide_attrs[gtx]['geolocation']['delta_time']['long_name'] = "Elapsed GPS seconds"
@@ -556,6 +625,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- delta time in geophys_corr group
         IS2_atl03_tide[gtx]['geophys_corr']['delta_time'] = delta_time
         IS2_atl03_fill[gtx]['geophys_corr']['delta_time'] = None
+        IS2_atl03_dims[gtx]['geophys_corr']['delta_time'] = None
         IS2_atl03_tide_attrs[gtx]['geophys_corr']['delta_time'] = {}
         IS2_atl03_tide_attrs[gtx]['geophys_corr']['delta_time']['units'] = "seconds since 2018-01-01"
         IS2_atl03_tide_attrs[gtx]['geophys_corr']['delta_time']['long_name'] = "Elapsed GPS seconds"
@@ -574,6 +644,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- latitude
         IS2_atl03_tide[gtx]['geolocation']['reference_photon_lat'] = lat
         IS2_atl03_fill[gtx]['geolocation']['reference_photon_lat'] = None
+        IS2_atl03_dims[gtx]['geolocation']['reference_photon_lat'] = ['delta_time']
         IS2_atl03_tide_attrs[gtx]['geolocation']['reference_photon_lat'] = {}
         IS2_atl03_tide_attrs[gtx]['geolocation']['reference_photon_lat']['units'] = "degrees_north"
         IS2_atl03_tide_attrs[gtx]['geolocation']['reference_photon_lat']['contentType'] = "physicalMeasurement"
@@ -588,6 +659,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- longitude
         IS2_atl03_tide[gtx]['geolocation']['reference_photon_lon'] = lon
         IS2_atl03_fill[gtx]['geolocation']['reference_photon_lon'] = None
+        IS2_atl03_dims[gtx]['geolocation']['reference_photon_lon'] = ['delta_time']
         IS2_atl03_tide_attrs[gtx]['geolocation']['reference_photon_lon'] = {}
         IS2_atl03_tide_attrs[gtx]['geolocation']['reference_photon_lon']['units'] = "degrees_east"
         IS2_atl03_tide_attrs[gtx]['geolocation']['reference_photon_lon']['contentType'] = "physicalMeasurement"
@@ -602,6 +674,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- segment ID
         IS2_atl03_tide[gtx]['geolocation']['segment_id'] = segment_id
         IS2_atl03_fill[gtx]['geolocation']['segment_id'] = None
+        IS2_atl03_dims[gtx]['geolocation']['segment_id'] = ['delta_time']
         IS2_atl03_tide_attrs[gtx]['geolocation']['segment_id'] = {}
         IS2_atl03_tide_attrs[gtx]['geolocation']['segment_id']['units'] = "1"
         IS2_atl03_tide_attrs[gtx]['geolocation']['segment_id']['contentType'] = "referenceInformation"
@@ -616,6 +689,7 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         #-- computed tide
         IS2_atl03_tide[gtx]['geophys_corr'][variable] = tide
         IS2_atl03_fill[gtx]['geophys_corr'][variable] = tide.fill_value
+        IS2_atl03_dims[gtx]['geophys_corr'][variable] = ['delta_time']
         IS2_atl03_tide_attrs[gtx]['geophys_corr'][variable] = {}
         IS2_atl03_tide_attrs[gtx]['geophys_corr'][variable]['units'] = "meters"
         IS2_atl03_tide_attrs[gtx]['geophys_corr'][variable]['contentType'] = "referenceInformation"
@@ -623,26 +697,22 @@ def compute_tides_ICESat2(tide_dir, FILE, TIDE_MODEL=None, METHOD='spline',
         IS2_atl03_tide_attrs[gtx]['geophys_corr'][variable]['description'] = description
         IS2_atl03_tide_attrs[gtx]['geophys_corr'][variable]['source'] = TIDE_MODEL
         IS2_atl03_tide_attrs[gtx]['geophys_corr'][variable]['reference'] = reference
-        IS2_atl03_tide_attrs[gtx]['geophys_corr'][variable]['coordinates'] = \
-            "segment_id delta_time reference_photon_lat reference_photon_lon"
         IS2_atl03_tide_attrs[gtx]['geophys_corr'][variable]['coordinates'] = \
             ("../geolocation/segment_id ../geolocation/delta_time "
             "../geolocation/reference_photon_lat ../geolocation/reference_photon_lon")
 
-    #-- output tidal HDF5 file
-    args = (PRD,TIDE_MODEL,YY,MM,DD,HH,MN,SS,TRK,CYCL,GRAN,RL,VERS,AUX)
-    file_format = '{0}_{1}_TIDES_{2}{3}{4}{5}{6}{7}_{8}{9}{10}_{11}_{12}{13}.h5'
     #-- print file information
-    print('\t{0}'.format(file_format.format(*args))) if VERBOSE else None
-    HDF5_ATL03_tide_write(IS2_atl03_tide, IS2_atl03_tide_attrs, CLOBBER=True,
-        INPUT=os.path.basename(FILE), FILL_VALUE=IS2_atl03_fill,
-        FILENAME=os.path.join(DIRECTORY,file_format.format(*args)))
+    print('\t{0}'.format(OUTPUT_FILE)) if VERBOSE else None
+    HDF5_ATL03_tide_write(IS2_atl03_tide, IS2_atl03_tide_attrs,
+        CLOBBER=True, INPUT=os.path.basename(INPUT_FILE),
+        FILL_VALUE=IS2_atl03_fill, DIMENSIONS=IS2_atl03_dims,
+        FILENAME=os.path.join(DIRECTORY,OUTPUT_FILE))
     #-- change the permissions mode
-    os.chmod(os.path.join(DIRECTORY,file_format.format(*args)), MODE)
+    os.chmod(os.path.join(DIRECTORY,OUTPUT_FILE), MODE)
 
 #-- PURPOSE: outputting the tide values for ICESat-2 data to HDF5
 def HDF5_ATL03_tide_write(IS2_atl03_tide, IS2_atl03_attrs, INPUT=None,
-    FILENAME='', FILL_VALUE=None, CLOBBER=False):
+    FILENAME='', FILL_VALUE=None, DIMENSIONS=None, CLOBBER=False):
     #-- setting HDF5 clobber attribute
     if CLOBBER:
         clobber = 'w'
@@ -685,33 +755,30 @@ def HDF5_ATL03_tide_write(IS2_atl03_tide, IS2_atl03_attrs, INPUT=None,
                 att_val = IS2_atl03_attrs[gtx][key][att_name]
                 fileID[gtx][key].attrs[att_name] = att_val
 
-            #-- delta_time in group
-            v = IS2_atl03_tide[gtx][key]['delta_time']
-            attrs = IS2_atl03_attrs[gtx][key]['delta_time']
-            fillvalue = FILL_VALUE[gtx][key]['delta_time']
-            #-- Defining the HDF5 dataset variables
-            val = '{0}/{1}/{2}'.format(gtx,key,'delta_time')
-            h5[gtx][key]['delta_time'] = fileID.create_dataset(val,np.shape(v),
-                data=v, dtype=v.dtype, fillvalue=fillvalue, compression='gzip')
-            #-- add HDF5 variable attributes
-            for att_name,att_val in attrs.items():
-                h5[gtx][key]['delta_time'].attrs[att_name] = att_val
-
-            #-- add other variables for group
+            #-- all variables for group
             groupkeys = set(IS2_atl03_tide[gtx][key].keys())-set(['delta_time'])
-            for k in sorted(groupkeys):
+            for k in ['delta_time',*sorted(groupkeys)]:
                 #-- values and attributes
                 v = IS2_atl03_tide[gtx][key][k]
                 attrs = IS2_atl03_attrs[gtx][key][k]
                 fillvalue = FILL_VALUE[gtx][key][k]
                 #-- Defining the HDF5 dataset variables
                 val = '{0}/{1}/{2}'.format(gtx,key,k)
-                h5[gtx][key][k] = fileID.create_dataset(val,np.shape(v),data=v,
-                    dtype=v.dtype, fillvalue=fillvalue, compression='gzip')
-                #-- attach dimensions
-                for dim in ['delta_time']:
-                    h5[gtx][key][k].dims.create_scale(h5[gtx][key][dim], dim)
-                    h5[gtx][key][k].dims[0].attach_scale(h5[gtx][key][dim])
+                if fillvalue:
+                    h5[gtx][key][k] = fileID.create_dataset(val, np.shape(v),
+                        data=v, dtype=v.dtype, fillvalue=fillvalue,
+                        compression='gzip')
+                else:
+                    h5[gtx][key][k] = fileID.create_dataset(val, np.shape(v),
+                        data=v, dtype=v.dtype, compression='gzip')
+                #-- create or attach dimensions for HDF5 variable
+                if DIMENSIONS[gtx][key][k]:
+                    #-- attach dimensions
+                    for i,dim in enumerate(DIMENSIONS[gtx][key][k]):
+                        h5[gtx][key][k].dims[i].attach_scale(h5[gtx][key][dim])
+                else:
+                    #-- make dimension
+                    h5[gtx][key][k].make_scale(k)
                 #-- add HDF5 variable attributes
                 for att_name,att_val in attrs.items():
                     h5[gtx][key][k].attrs[att_name] = att_val
@@ -735,9 +802,9 @@ def HDF5_ATL03_tide_write(IS2_atl03_tide, IS2_atl03_attrs, INPUT=None,
     instrument = 'ATLAS > Advanced Topographic Laser Altimeter System'
     fileID.attrs['instrument'] = instrument
     fileID.attrs['source'] = 'Spacecraft'
-    fileID.attrs['references'] = 'http://nsidc.org/data/icesat2/data.html'
+    fileID.attrs['references'] = 'https://nsidc.org/data/icesat-2'
     fileID.attrs['processing_level'] = '4'
-    #-- add attributes for input ATL03 files
+    #-- add attributes for input ATL03 file
     fileID.attrs['input_files'] = os.path.basename(INPUT)
     #-- find geospatial and temporal ranges
     lnmn,lnmx,ltmn,ltmx,tmn,tmx = (np.inf,-np.inf,np.inf,-np.inf,np.inf,-np.inf)
@@ -770,14 +837,14 @@ def HDF5_ATL03_tide_write(IS2_atl03_tide, IS2_atl03_attrs, INPUT=None,
     #-- convert from seconds since 1980-01-06T00:00:00 to Julian days
     time_julian = 2400000.5 + pyTMD.time.convert_delta_time(gps_seconds - leaps,
         epoch1=(1980,1,6,0,0,0), epoch2=(1858,11,17,0,0,0), scale=1.0/86400.0)
-    #-- convert to calendar date with convert_julian.py
-    YY,MM,DD,HH,MN,SS = convert_julian(time_julian,FORMAT='tuple')
+    #-- convert to calendar date
+    YY,MM,DD,HH,MN,SS = pyTMD.time.convert_julian(time_julian,FORMAT='tuple')
     #-- add attributes with measurement date start, end and duration
-    tcs = datetime.datetime(np.int(YY[0]), np.int(MM[0]), np.int(DD[0]),
-        np.int(HH[0]), np.int(MN[0]), np.int(SS[0]), np.int(1e6*(SS[0] % 1)))
+    tcs = datetime.datetime(int(YY[0]), int(MM[0]), int(DD[0]),
+        int(HH[0]), int(MN[0]), int(SS[0]), int(1e6*(SS[0] % 1)))
     fileID.attrs['time_coverage_start'] = tcs.isoformat()
-    tce = datetime.datetime(np.int(YY[1]), np.int(MM[1]), np.int(DD[1]),
-        np.int(HH[1]), np.int(MN[1]), np.int(SS[1]), np.int(1e6*(SS[1] % 1)))
+    tce = datetime.datetime(int(YY[1]), int(MM[1]), int(DD[1]),
+        int(HH[1]), int(MN[1]), int(SS[1]), int(1e6*(SS[1] % 1)))
     fileID.attrs['time_coverage_end'] = tce.isoformat()
     fileID.attrs['time_coverage_duration'] = '{0:0.0f}'.format(tmx-tmn)
     #-- Closing the HDF5 file
@@ -802,7 +869,7 @@ def main():
         help='Working data directory')
     #-- tide model to use
     model_choices = ('CATS0201','CATS2008','CATS2008_load',
-        'TPXO9-atlas','TPXO9-atlas-v2','TPXO9-atlas-v3',
+        'TPXO9-atlas','TPXO9-atlas-v2','TPXO9-atlas-v3','TPXO9-atlas-v4',
         'TPXO9.1','TPXO8-atlas','TPXO7.2','TPXO7.2_load',
         'AODTM-5','AOTIM-5','AOTIM-5-2018',
         'GOT4.7','GOT4.7_load','GOT4.8','GOT4.8_load','GOT4.10','GOT4.10_load',
@@ -816,6 +883,10 @@ def main():
         metavar='METHOD', type=str, default='spline',
         choices=('spline','linear','nearest','bilinear'),
         help='Spatial interpolation method')
+    #-- extrapolate with nearest-neighbors
+    parser.add_argument('--extrapolate','-E',
+        default=False, action='store_true',
+        help='Extrapolate with nearest-neighbors')
     #-- verbosity settings
     #-- verbose will output information about each output file
     parser.add_argument('--verbose','-V',
@@ -830,7 +901,8 @@ def main():
     #-- run for each input ATL03 file
     for FILE in args.infile:
         compute_tides_ICESat2(args.directory, FILE, TIDE_MODEL=args.tide,
-            METHOD=args.interpolate, VERBOSE=args.verbose, MODE=args.mode)
+            METHOD=args.interpolate, EXTRAPOLATE=args.extrapolate,
+            VERBOSE=args.verbose, MODE=args.mode)
 
 #-- run main program
 if __name__ == '__main__':
