@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 u"""
 compute_tides_icebridge_data.py
-Written by Tyler Sutterley (11/2020)
+Written by Tyler Sutterley (03/2021)
 Calculates tidal elevations for correcting Operation IceBridge elevation data
 
 Uses OTIS format tidal solutions provided by Ohio State University and ESR
@@ -23,6 +23,7 @@ COMMAND LINE OPTIONS:
         TPXO9-atlas
         TPXO9-atlas-v2
         TPXO9-atlas-v3
+        TPXO9-atlas-v4
         TPXO9.1
         TPXO8-atlas
         TPXO7.2
@@ -43,6 +44,7 @@ COMMAND LINE OPTIONS:
         linear
         nearest
         bilinear
+    -E X, --extrapolate X: Extrapolate with nearest-neighbors
     -M X, --mode X: Permission mode of directories and files created
     -V, --verbose: Output information about each created file
 
@@ -62,21 +64,27 @@ PYTHON DEPENDENCIES:
 PROGRAM DEPENDENCIES:
     time.py: utilities for calculating time operations
     utilities: download and management utilities for syncing files
-    convert_julian.py: returns the calendar date and time given a Julian date
     calc_astrol_longitudes.py: computes the basic astronomical mean longitudes
     calc_delta_time.py: calculates difference between universal and dynamic time
     convert_ll_xy.py: convert lat/lon points to and from projected coordinates
     infer_minor_corrections.py: return corrections for minor constituents
     load_constituent.py: loads parameters for a given tidal constituent
     load_nodal_corrections.py: load the nodal corrections for tidal constituents
-    predict_tide_drift.py: predict tidal elevations using harmonic constants
     read_tide_model.py: extract tidal harmonic constants from OTIS tide models
     read_netcdf_model.py: extract tidal harmonic constants from netcdf models
     read_GOT_model.py: extract tidal harmonic constants from GSFC GOT models
     read_FES_model.py: extract tidal harmonic constants from FES tide models
+    bilinear_interp.py: bilinear interpolation of data to coordinates
+    nearest_extrap.py: nearest-neighbor extrapolation of data to coordinates
+    predict_tide_drift.py: predict tidal elevations using harmonic constants
     read_ATM1b_QFIT_binary.py: read ATM1b QFIT binary files (NSIDC version 1)
 
 UPDATE HISTORY:
+    Updated 03/2021: added TPXO9-atlas-v4 in binary OTIS format
+        simplified netcdf inputs to be similar to binary OTIS read program
+        replaced numpy bool/int to prevent deprecation warnings
+    Updated 12/2020: added valid data extrapolation with nearest_extrap
+         merged time conversion routines into module
     Updated 11/2020: added model constituents from TPXO9-atlas-v3
     Updated 10/2020: using argparse to set command line parameters
     Updated 09/2020: output ocean and load tide as tide_ocean and tide_load
@@ -105,7 +113,6 @@ import argparse
 import numpy as np
 import pyTMD.time
 from pyTMD.utilities import get_data_path
-from pyTMD.convert_julian import convert_julian
 from read_ATM1b_QFIT_binary.read_ATM1b_QFIT_binary import read_ATM1b_QFIT_binary
 from pyTMD.calc_delta_time import calc_delta_time
 from pyTMD.infer_minor_corrections import infer_minor_corrections
@@ -164,22 +171,22 @@ def read_ATM_qfit_file(input_file, input_subsetter):
         #-- number of lines of data within file
         file_lines = file_length(input_file,input_subsetter)
         #-- create output variables with length equal to the number of lines
-        ATM_L1b_input['lat'] = np.zeros_like(file_contents,dtype=np.float)
-        ATM_L1b_input['lon'] = np.zeros_like(file_contents,dtype=np.float)
-        ATM_L1b_input['data'] = np.zeros_like(file_contents,dtype=np.float)
-        hour = np.zeros_like(file_contents,dtype=np.float)
-        minute = np.zeros_like(file_contents,dtype=np.float)
-        second = np.zeros_like(file_contents,dtype=np.float)
+        ATM_L1b_input['lat'] = np.zeros_like(file_contents,dtype=np.float64)
+        ATM_L1b_input['lon'] = np.zeros_like(file_contents,dtype=np.float64)
+        ATM_L1b_input['data'] = np.zeros_like(file_contents,dtype=np.float64)
+        hour = np.zeros_like(file_contents,dtype=np.float64)
+        minute = np.zeros_like(file_contents,dtype=np.float64)
+        second = np.zeros_like(file_contents,dtype=np.float64)
         #-- for each line within the file
         for i,line in enumerate(file_contents):
             #-- find numerical instances within the line
             line_contents = rx.findall(line)
-            ATM_L1b_input['lat'][i] = np.float(line_contents[1])
-            ATM_L1b_input['lon'][i] = np.float(line_contents[2])
-            ATM_L1b_input['data'][i] = np.float(line_contents[3])
-            hour[i] = np.float(line_contents[-1][:2])
-            minute[i] = np.float(line_contents[-1][2:4])
-            second[i] = np.float(line_contents[-1][4:])
+            ATM_L1b_input['lat'][i] = np.float64(line_contents[1])
+            ATM_L1b_input['lon'][i] = np.float64(line_contents[2])
+            ATM_L1b_input['data'][i] = np.float64(line_contents[3])
+            hour[i] = np.float64(line_contents[-1][:2])
+            minute[i] = np.float64(line_contents[-1][2:4])
+            second[i] = np.float64(line_contents[-1][4:])
     #-- Version 1 of ATM QFIT files (binary)
     elif (SFX == 'qi'):
         #-- read input QFIT data file and subset if specified
@@ -191,16 +198,16 @@ def read_ATM_qfit_file(input_file, input_subsetter):
         ATM_L1b_input['data'] = fid['elevation'][:]
         time_hhmmss = fid['time_hhmmss'][:]
         #-- extract hour, minute and second from time_hhmmss
-        hour = np.zeros_like(time_hhmmss,dtype=np.float)
-        minute = np.zeros_like(time_hhmmss,dtype=np.float)
-        second = np.zeros_like(time_hhmmss,dtype=np.float)
+        hour = np.zeros_like(time_hhmmss,dtype=np.float64)
+        minute = np.zeros_like(time_hhmmss,dtype=np.float64)
+        second = np.zeros_like(time_hhmmss,dtype=np.float64)
         #-- for each line within the file
         for i,packed_time in enumerate(time_hhmmss):
             #-- convert to zero-padded string with 3 decimal points
             line_contents = '{0:010.3f}'.format(packed_time)
-            hour[i] = np.float(line_contents[:2])
-            minute[i] = np.float(line_contents[2:4])
-            second[i] = np.float(line_contents[4:])
+            hour[i] = np.float64(line_contents[:2])
+            minute[i] = np.float64(line_contents[2:4])
+            second[i] = np.float64(line_contents[4:])
     #-- Version 2 of ATM QFIT files (HDF5)
     elif (SFX == 'h5'):
         #-- Open the HDF5 file for reading
@@ -213,16 +220,16 @@ def read_ATM_qfit_file(input_file, input_subsetter):
         ATM_L1b_input['data'] = fileID['elevation'][:]
         time_hhmmss = fileID['instrument_parameters']['time_hhmmss'][:]
         #-- extract hour, minute and second from time_hhmmss
-        hour = np.zeros_like(time_hhmmss,dtype=np.float)
-        minute = np.zeros_like(time_hhmmss,dtype=np.float)
-        second = np.zeros_like(time_hhmmss,dtype=np.float)
+        hour = np.zeros_like(time_hhmmss,dtype=np.float64)
+        minute = np.zeros_like(time_hhmmss,dtype=np.float64)
+        second = np.zeros_like(time_hhmmss,dtype=np.float64)
         #-- for each line within the file
         for i,packed_time in enumerate(time_hhmmss):
             #-- convert to zero-padded string with 3 decimal points
             line_contents = '{0:010.3f}'.format(packed_time)
-            hour[i] = np.float(line_contents[:2])
-            minute[i] = np.float(line_contents[2:4])
-            second[i] = np.float(line_contents[4:])
+            hour[i] = np.float64(line_contents[:2])
+            minute[i] = np.float64(line_contents[2:4])
+            second[i] = np.float64(line_contents[4:])
         #-- close the input HDF5 file
         fileID.close()
     #-- calculate the number of leap seconds between GPS time (seconds
@@ -277,14 +284,14 @@ def read_ATM_icessn_file(input_file, input_subsetter):
     ATM_L2_input = {}
     #-- create output variables with length equal to the number of file lines
     for key in file_dtype.keys():
-        ATM_L2_input[key] = np.zeros_like(file_contents, dtype=np.float)
+        ATM_L2_input[key] = np.zeros_like(file_contents, dtype=np.float64)
     #-- for each line within the file
     for line_number,line_entries in enumerate(file_contents):
         #-- find numerical instances within the line
         line_contents = rx.findall(line_entries)
         #-- for each variable of interest: save to dinput as float
         for key,val in file_dtype.items():
-            ATM_L2_input[key][line_number] = np.float(line_contents[val])
+            ATM_L2_input[key][line_number] = np.float64(line_contents[val])
     #-- convert shot time (seconds of day) to J2000
     hour = np.floor(ATM_L2_input['seconds']/3600.0)
     minute = np.floor((ATM_L2_input['seconds'] % 3600)/60.0)
@@ -329,7 +336,7 @@ def read_LVIS_HDF5_file(input_file, input_subsetter):
     regex_pattern = r'{0}_(.*?)(\d+)_(\d+)_(R\d+)_(\d+).H5'.format(mission_flag)
     #-- extract mission, region and other parameters from filename
     MISSION,REGION,YY,MMDD,RLD,SS = re.findall(regex_pattern,input_file).pop()
-    LDS_VERSION = '2.0.2' if (np.int(RLD[1:3]) >= 18) else '1.04'
+    LDS_VERSION = '2.0.2' if (int(RLD[1:3]) >= 18) else '1.04'
     #-- input and output python dictionaries with variables
     file_input = {}
     LVIS_L2_input = {}
@@ -374,10 +381,10 @@ def read_LVIS_HDF5_file(input_file, input_subsetter):
     #-- close the input HDF5 file
     fileID.close()
     #-- output combined variables
-    LVIS_L2_input['data'] = np.zeros_like(file_input['elev'],dtype=np.float)
-    LVIS_L2_input['lon'] = np.zeros_like(file_input['elev'],dtype=np.float)
-    LVIS_L2_input['lat'] = np.zeros_like(file_input['elev'],dtype=np.float)
-    LVIS_L2_input['error'] = np.zeros_like(file_input['elev'],dtype=np.float)
+    LVIS_L2_input['data'] = np.zeros_like(file_input['elev'],dtype=np.float64)
+    LVIS_L2_input['lon'] = np.zeros_like(file_input['elev'],dtype=np.float64)
+    LVIS_L2_input['lat'] = np.zeros_like(file_input['elev'],dtype=np.float64)
+    LVIS_L2_input['error'] = np.zeros_like(file_input['elev'],dtype=np.float64)
     #-- find where elev high is equal to elev low
     #-- see note about using LVIS centroid elevation product
     #-- http://lvis.gsfc.nasa.gov/OIBDataStructure.html
@@ -411,7 +418,7 @@ def read_LVIS_HDF5_file(input_file, input_subsetter):
 #-- PURPOSE: read Operation IceBridge data from NSIDC
 #-- compute tides at points and times using tidal model driver algorithms
 def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
-    METHOD='spline', VERBOSE=False, MODE=0o775):
+    METHOD='spline', EXTRAPOLATE=False, VERBOSE=False, MODE=0o775):
 
     #-- extract file name and subsetter indices lists
     match_object = re.match(r'(.*?)(\[(.*?)\])?$',arg)
@@ -470,37 +477,41 @@ def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
         TYPE = 'z'
     elif (TIDE_MODEL == 'TPXO9-atlas'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas')
-        grid_file = 'grid_tpxo9_atlas.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30.nc.gz','h_o1_tpxo9_atlas_30.nc.gz',
             'h_p1_tpxo9_atlas_30.nc.gz','h_k1_tpxo9_atlas_30.nc.gz',
             'h_n2_tpxo9_atlas_30.nc.gz','h_m2_tpxo9_atlas_30.nc.gz',
             'h_s2_tpxo9_atlas_30.nc.gz','h_k2_tpxo9_atlas_30.nc.gz',
             'h_m4_tpxo9_atlas_30.nc.gz','h_ms4_tpxo9_atlas_30.nc.gz',
             'h_mn4_tpxo9_atlas_30.nc.gz','h_2n2_tpxo9_atlas_30.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = 'http://volkov.oce.orst.edu/tides/tpxo9_atlas.html'
         output_variable = 'tide_ocean'
         variable_long_name = 'Ocean_Tide'
         model_format = 'netcdf'
         TYPE = 'z'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'TPXO9-atlas-v2'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas_v2')
-        grid_file = 'grid_tpxo9_atlas_30_v2.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v2.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30_v2.nc.gz','h_o1_tpxo9_atlas_30_v2.nc.gz',
             'h_p1_tpxo9_atlas_30_v2.nc.gz','h_k1_tpxo9_atlas_30_v2.nc.gz',
             'h_n2_tpxo9_atlas_30_v2.nc.gz','h_m2_tpxo9_atlas_30_v2.nc.gz',
             'h_s2_tpxo9_atlas_30_v2.nc.gz','h_k2_tpxo9_atlas_30_v2.nc.gz',
             'h_m4_tpxo9_atlas_30_v2.nc.gz','h_ms4_tpxo9_atlas_30_v2.nc.gz',
             'h_mn4_tpxo9_atlas_30_v2.nc.gz','h_2n2_tpxo9_atlas_30_v2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = 'https://www.tpxo.net/global/tpxo9-atlas'
         output_variable = 'tide_ocean'
         variable_long_name = 'Ocean_Tide'
         model_format = 'netcdf'
         TYPE = 'z'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'TPXO9-atlas-v3'):
         model_directory = os.path.join(tide_dir,'TPXO9_atlas_v3')
-        grid_file = 'grid_tpxo9_atlas_30_v3.nc.gz'
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v3.nc.gz')
         model_files = ['h_q1_tpxo9_atlas_30_v3.nc.gz','h_o1_tpxo9_atlas_30_v3.nc.gz',
             'h_p1_tpxo9_atlas_30_v3.nc.gz','h_k1_tpxo9_atlas_30_v3.nc.gz',
             'h_n2_tpxo9_atlas_30_v3.nc.gz','h_m2_tpxo9_atlas_30_v3.nc.gz',
@@ -508,12 +519,31 @@ def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
             'h_m4_tpxo9_atlas_30_v3.nc.gz','h_ms4_tpxo9_atlas_30_v3.nc.gz',
             'h_mn4_tpxo9_atlas_30_v3.nc.gz','h_2n2_tpxo9_atlas_30_v3.nc.gz',
             'h_mf_tpxo9_atlas_30_v3.nc.gz','h_mm_tpxo9_atlas_30_v3.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = 'https://www.tpxo.net/global/tpxo9-atlas'
         output_variable = 'tide_ocean'
         variable_long_name = "Ocean Tide"
         model_format = 'netcdf'
         TYPE = 'z'
         SCALE = 1.0/1000.0
+        GZIP = True
+    elif (TIDE_MODEL == 'TPXO9-atlas-v4'):
+        model_directory = os.path.join(tide_dir,'TPXO9_atlas_v4')
+        grid_file = os.path.join(model_directory,'grid_tpxo9_atlas_30_v4')
+        model_files = ['h_q1_tpxo9_atlas_30_v4','h_o1_tpxo9_atlas_30_v4',
+            'h_p1_tpxo9_atlas_30_v4','h_k1_tpxo9_atlas_30_v4',
+            'h_n2_tpxo9_atlas_30_v4','h_m2_tpxo9_atlas_30_v4',
+            'h_s2_tpxo9_atlas_30_v4','h_k2_tpxo9_atlas_30_v4',
+            'h_m4_tpxo9_atlas_30_v4','h_ms4_tpxo9_atlas_30_v4',
+            'h_mn4_tpxo9_atlas_30_v4','h_2n2_tpxo9_atlas_30_v4',
+            'h_mf_tpxo9_atlas_30_v4','h_mm_tpxo9_atlas_30_v4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
+        reference = 'https://www.tpxo.net/global/tpxo9-atlas'
+        output_variable = 'tide_ocean'
+        variable_long_name = 'Ocean_Tide'
+        model_format = 'OTIS'
+        EPSG = '4326'
+        TYPE = 'z'
     elif (TIDE_MODEL == 'TPXO9.1'):
         grid_file = os.path.join(tide_dir,'TPXO9.1','DATA','grid_tpxo9')
         model_file = os.path.join(tide_dir,'TPXO9.1','DATA','h_tpxo9.v1')
@@ -584,71 +614,77 @@ def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         output_variable = 'tide_ocean'
         variable_long_name = 'Ocean_Tide'
         model_format = 'GOT'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.7_load'):
         model_directory = os.path.join(tide_dir,'GOT4.7','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         output_variable = 'tide_load'
         variable_long_name = 'Load_Tide'
         model_format = 'GOT'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.8'):
         model_directory = os.path.join(tide_dir,'got4.8','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         output_variable = 'tide_ocean'
         variable_long_name = 'Ocean_Tide'
         model_format = 'GOT'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.8_load'):
         model_directory = os.path.join(tide_dir,'got4.8','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         output_variable = 'tide_load'
         variable_long_name = 'Load_Tide'
         model_format = 'GOT'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.10'):
         model_directory = os.path.join(tide_dir,'GOT4.10c','grids_oceantide')
         model_files = ['q1.d.gz','o1.d.gz','p1.d.gz','k1.d.gz','n2.d.gz',
             'm2.d.gz','s2.d.gz','k2.d.gz','s1.d.gz','m4.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         output_variable = 'tide_ocean'
         variable_long_name = 'Ocean_Tide'
         model_format = 'GOT'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'GOT4.10_load'):
         model_directory = os.path.join(tide_dir,'GOT4.10c','grids_loadtide')
         model_files = ['q1load.d.gz','o1load.d.gz','p1load.d.gz','k1load.d.gz',
             'n2load.d.gz','m2load.d.gz','s2load.d.gz','k2load.d.gz',
             's1load.d.gz','m4load.d.gz']
-        c = ['q1','o1','p1','k1','n2','m2','s2','k2','s1','m4']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         reference = ('https://denali.gsfc.nasa.gov/personal_pages/ray/'
             'MiscPubs/19990089548_1999150788.pdf')
         output_variable = 'tide_load'
         variable_long_name = 'Load_Tide'
         model_format = 'GOT'
         SCALE = 1.0/1000.0
+        GZIP = True
     elif (TIDE_MODEL == 'FES2014'):
         model_directory = os.path.join(tide_dir,'fes2014','ocean_tide')
         model_files = ['2n2.nc.gz','eps2.nc.gz','j1.nc.gz','k1.nc.gz',
@@ -658,16 +694,18 @@ def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
             'mu2.nc.gz','n2.nc.gz','n4.nc.gz','nu2.nc.gz','o1.nc.gz','p1.nc.gz',
             'q1.nc.gz','r2.nc.gz','s1.nc.gz','s2.nc.gz','s4.nc.gz','sa.nc.gz',
             'ssa.nc.gz','t2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['2n2','eps2','j1','k1','k2','l2','lambda2','m2','m3','m4','m6',
             'm8','mf','mks2','mm','mn4','ms4','msf','msqm','mtm','mu2','n2',
             'n4','nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
-        reference = ('https://www.aviso.altimetry.fr/data/products/'
+        reference = ('https://www.aviso.altimetry.fr/en/data/products'
             'auxiliary-products/global-tide-fes.html')
         output_variable = 'tide_ocean'
         variable_long_name = 'Ocean_Tide'
         model_format = 'FES'
         TYPE = 'z'
         SCALE = 1.0/100.0
+        GZIP = True
     elif (TIDE_MODEL == 'FES2014_load'):
         model_directory = os.path.join(tide_dir,'fes2014','load_tide')
         model_files = ['2n2.nc.gz','eps2.nc.gz','j1.nc.gz','k1.nc.gz',
@@ -677,16 +715,18 @@ def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
             'mu2.nc.gz','n2.nc.gz','n4.nc.gz','nu2.nc.gz','o1.nc.gz','p1.nc.gz',
             'q1.nc.gz','r2.nc.gz','s1.nc.gz','s2.nc.gz','s4.nc.gz','sa.nc.gz',
             'ssa.nc.gz','t2.nc.gz']
+        model_file = [os.path.join(model_directory,m) for m in model_files]
         c = ['2n2','eps2','j1','k1','k2','l2','lambda2','m2','m3','m4','m6',
             'm8','mf','mks2','mm','mn4','ms4','msf','msqm','mtm','mu2','n2',
             'n4','nu2','o1','p1','q1','r2','s1','s2','s4','sa','ssa','t2']
-        reference = ('https://www.aviso.altimetry.fr/data/products/'
+        reference = ('https://www.aviso.altimetry.fr/en/data/products'
             'auxiliary-products/global-tide-fes.html')
         output_variable = 'tide_load'
         variable_long_name = 'Load_Tide'
         model_format = 'FES'
         TYPE = 'z'
         SCALE = 1.0/100.0
+        GZIP = True
 
     #-- HDF5 file attributes
     attrib = {}
@@ -727,10 +767,10 @@ def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
         #-- early date strings omitted century and millenia (e.g. 93 for 1993)
         if (len(YYMMDD1) == 6):
             ypre,MM1,DD1 = YYMMDD1[:2],YYMMDD1[2:4],YYMMDD1[4:]
-            if (np.float(ypre) >= 90):
-                YY1 = '{0:4.0f}'.format(np.float(ypre) + 1900.0)
+            if (np.float64(ypre) >= 90):
+                YY1 = '{0:4.0f}'.format(np.float64(ypre) + 1900.0)
             else:
-                YY1 = '{0:4.0f}'.format(np.float(ypre) + 2000.0)
+                YY1 = '{0:4.0f}'.format(np.float64(ypre) + 2000.0)
         elif (len(YYMMDD1) == 8):
             YY1,MM1,DD1 = YYMMDD1[:4],YYMMDD1[4:6],YYMMDD1[6:]
     elif OIB in ('LVIS','LVGH'):
@@ -754,30 +794,31 @@ def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
     t = pyTMD.time.convert_delta_time(dinput['time'],
         epoch1=(2000,1,1,12,0,0), epoch2=(1992,1,1,0,0,0),
         scale=1.0/86400.0)
+    #-- delta time (TT - UT1) file
+    delta_file = get_data_path(['data','merged_deltat.data'])
 
     #-- read tidal constants and interpolate to grid points
     if model_format in ('OTIS','ATLAS'):
         amp,ph,D,c = extract_tidal_constants(dinput['lon'], dinput['lat'],
             grid_file, model_file, EPSG, TYPE=TYPE, METHOD=METHOD,
-            GRID=model_format)
+            EXTRAPOLATE=EXTRAPOLATE, GRID=model_format)
         deltat = np.zeros_like(t)
     elif model_format in ('netcdf'):
         amp,ph,D,c = extract_netcdf_constants(dinput['lon'], dinput['lat'],
-            model_directory, grid_file, model_files, TYPE=TYPE, METHOD=METHOD,
-            SCALE=SCALE)
+            grid_file, model_file, TYPE=TYPE, METHOD=METHOD,
+            EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE, GZIP=GZIP)
         deltat = np.zeros_like(t)
     elif (model_format == 'GOT'):
-        amp,ph = extract_GOT_constants(dinput['lon'], dinput['lat'],
-            model_directory, model_files, METHOD=METHOD, SCALE=SCALE)
+        amp,ph,c = extract_GOT_constants(dinput['lon'], dinput['lat'],
+            model_file, METHOD=METHOD, EXTRAPOLATE=EXTRAPOLATE,
+            SCALE=SCALE, GZIP=GZIP)
         #-- interpolate delta times from calendar dates to tide time
-        delta_file = get_data_path(['data','merged_deltat.data'])
         deltat = calc_delta_time(delta_file, t)
     elif (model_format == 'FES'):
         amp,ph = extract_FES_constants(dinput['lon'], dinput['lat'],
-            model_directory, model_files, TYPE=TYPE, VERSION=TIDE_MODEL,
-            METHOD=METHOD, SCALE=SCALE)
+            model_file, TYPE=TYPE, VERSION=TIDE_MODEL, METHOD=METHOD,
+            EXTRAPOLATE=EXTRAPOLATE, SCALE=SCALE, GZIP=GZIP)
         #-- interpolate delta times from calendar dates to tide time
-        delta_file = get_data_path(['data','merged_deltat.data'])
         deltat = calc_delta_time(delta_file, t)
 
     #-- calculate complex phase in radians for Euler's
@@ -856,7 +897,7 @@ def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
     fid.attrs['project'] = 'NASA_Operation_IceBridge'
     fid.attrs['processing_level'] = '4'
     fid.attrs['date_created'] = time.strftime('%Y-%m-%d',time.localtime())
-    #-- add attributes for input files
+    #-- add attributes for input file
     fid.attrs['elevation_file'] = os.path.basename(input_file)
     fid.attrs['tide_model'] = TIDE_MODEL
     #-- add geospatial and temporal attributes
@@ -872,8 +913,8 @@ def compute_tides_icebridge_data(tide_dir, arg, TIDE_MODEL,
     time_range = np.array([np.min(t),np.max(t)])
     time_julian = 2400000.5 + pyTMD.time.convert_delta_time(time_range,
         epoch1=(1992,1,1,0,0,0), epoch2=(1858,11,17,0,0,0), scale=1.0)
-    #-- convert to calendar date with convert_julian.py
-    cal = convert_julian(time_julian,ASTYPE=np.int)
+    #-- convert to calendar date
+    cal = pyTMD.time.convert_julian(time_julian,ASTYPE=int)
     #-- add attributes with measurement date start, end and duration
     args = (cal['hour'][0],cal['minute'][0],cal['second'][0])
     fid.attrs['RangeBeginningTime'] = '{0:02d}:{1:02d}:{2:02d}'.format(*args)
@@ -909,7 +950,7 @@ def main():
         help='Working data directory')
     #-- tide model to use
     model_choices = ('CATS0201','CATS2008','CATS2008_load',
-        'TPXO9-atlas','TPXO9-atlas-v2','TPXO9-atlas-v3',
+        'TPXO9-atlas','TPXO9-atlas-v2','TPXO9-atlas-v3','TPXO9-atlas-v4',
         'TPXO9.1','TPXO8-atlas','TPXO7.2','TPXO7.2_load',
         'AODTM-5','AOTIM-5','AOTIM-5-2018',
         'GOT4.7','GOT4.7_load','GOT4.8','GOT4.8_load','GOT4.10','GOT4.10_load',
@@ -923,6 +964,10 @@ def main():
         metavar='METHOD', type=str, default='spline',
         choices=('spline','linear','nearest','bilinear'),
         help='Spatial interpolation method')
+    #-- extrapolate with nearest-neighbors
+    parser.add_argument('--extrapolate','-E',
+        default=False, action='store_true',
+        help='Extrapolate with nearest-neighbors')
     #-- verbosity settings
     #-- verbose will output information about each output file
     parser.add_argument('--verbose','-V',
@@ -937,7 +982,8 @@ def main():
     #-- run for each input Operation IceBridge file
     for arg in args.infile:
         compute_tides_icebridge_data(args.directory, arg, TIDE_MODEL=args.tide,
-            METHOD=args.interpolate, VERBOSE=args.verbose, MODE=args.mode)
+            METHOD=args.interpolate, EXTRAPOLATE=args.extrapolate,
+            VERBOSE=args.verbose, MODE=args.mode)
 
 #-- run main program
 if __name__ == '__main__':
